/**
 * 数据库表格
 */

import * as MetaInit from '../../../../pi/struct/meta_init';
import { TabMeta, FieldInfo } from '../../../../pi/struct/sinfo';
import { read as bonRead, write } from '../../../../pi/struct/util';
import { BonBuffer } from '../../../../pi/util/bon';
import { Forelet } from '../../../../pi/widget/forelet';
import { Widget } from '../../../../pi/widget/widget';
import { AuthEnum } from '../../../server/data/db/auth.s';
import { DBOperation } from '../../../server/rpc/db_mgr.s';
import { newPrimaryKeyType, newValueFromJsonString } from '../../../utils/util';
import { decodeTableStruct, encodeTableStruct, iterDb, metaInfo, tables, wares, writeDb } from '../../net/rpc';
import { getStore } from '../../store/memstore';
import { equal, popNewMessage } from '../../util/logic';

MetaInit.init();
const structMgr = MetaInit.mgr;
export const forelet = new Forelet();
declare var pi_modules;
// tslint:disable-next-line:no-reserved-keywords
declare var module: any;
export const WIDGET_NAME = module.id.replace(/\//g, '-');

interface Props {
    showTitleList:any[];// 表格标题
    showDataList:any[];// 显示表格值
    datas:any[];  // 实际数据
    allCheck:boolean;// 是否全选 1全选 0全不选
    searchString:string;// 搜索的字段
    database:any[];// 数据库列表
    currentTable:any[];// 表
    currentDatabaseName:string;// 当前数据库
    currentTableName:string;// 当前数据表
    currentIndex:number;// 当前页数
    primary:any;// 主键名
    changeNum:number;// 修改某条数据
    addData:boolean; // 新增数据
    pages:number;
    curPage:number;
    showList:boolean[]; // 下拉框显示
    WPower:boolean;  // 操作数据权限
    fields: any;
    notes: any;
}

// tslint:disable-next-line:completed-docs
export class Table extends Widget {
    public ok:() => void;
    public props:Props = {
        showTitleList:[],
        showDataList:[],
        datas:[],
        allCheck :false,
        searchString:'',
        database:[],
        currentTable:[],
        currentDatabaseName:'',
        currentTableName:'',
        currentIndex:0,
        primary:null,
        changeNum:-1,
        addData:false,
        pages:1,
        curPage:0,
        showList:[false,false],
        WPower:false,
        fields: undefined,
        notes: undefined,
    };

    public create() {
        super.create();
        wares(r => {
            this.props.database = r;
        });
        // this.props.database = ["file", "memory"];
        // const power = getStore('ownerPower',[]);
        const power = [0,1,2,3,4,5,6];
        if (power.indexOf(AuthEnum.DBRW) > -1) {
            this.props.WPower = true;
        }

    }

    /**
     * 获取表头
     */
    public getTableTitle() {
        metaInfo(this.props.currentDatabaseName, this.props.currentTableName, r => {
            console.log('getMetaInfo: ', r);
            const obj = JSON.parse(r);
            // 判断是否存在主键
            if (obj.notes) {
                this.props.notes = obj.notes;
                for (const [k, cf] of obj.notes) {
                    if (k === 'primary') {
                        this.props.primary = cf;
                    }
        
                }
            } else {
                this.props.primary = undefined;
            }
            if (obj.fields) {
                this.props.fields = obj.fields;
            }
            this.props.showTitleList = [];
            obj.fields.forEach((v) => {
                this.props.showTitleList.push(v.name);
            });
            this.getTableData(100);
        });
    }
    
    /**
     * 获取表数据
     * @param index 获取数据库前多少个数据
     * @param startKey 主键位置
     */
    public getTableData(index:number,startKey?:any) {
        console.log('getTableData --- index, startkey ', index, startKey);
        iterDb(this.props.currentDatabaseName, this.props.currentTableName, index, startKey, null, false,r => {
            const b = new BonBuffer(r.msg);
            const x = b.readArray(() => {
                return b.readBin();
            });
            
            decodeTableStruct(this.props.currentTableName, x, r => {
                const datas = [];
                for (const v of r.res) {
                    datas.push(JSON.parse(v));
                }
                this.props.datas = datas;
                this.props.showDataList = datas;
                this.props.pages = Math.floor(this.props.showDataList.length / 10) + 1;
                this.paint();
            });
        });
    }

    // 切换数据库
    public databaseChange(e:any) {
        console.log(e.value);
        const ware = e.value;
        this.props.currentDatabaseName = ware;
        tables(ware, r => {
            console.log('get tabs: ', r);
            this.props.currentTable = r;
            this.paint();
        });
    }
    
    // 切换表
    public tableChange(e:any) {
        this.props.currentTableName = e.value;
        this.getTableTitle();
        this.paint();
    }
    
    // 搜索框输入
    public searchChange(e:any) {
        this.props.searchString = e.value;
    }
    
    // 搜索
    public search() {
        if (this.props.searchString) {
            const [key,val] = this.props.searchString.split('=');
            this.props.showDataList = this.props.datas.filter(item => {
                return val ? item[key].toString() === val :true; // 输入内容为空也匹配全部
            });
        } else {
            this.props.showDataList = this.props.datas;
        }
        
        this.paint();
    }
    
    // 分页触发函数
    public changeCurrent(e:any) {
        this.props.curPage = e.value;
        this.paint();
    }

    // 保存修改
    public changeData(e:any) {
        e.value[this.props.primary] = this.props.datas[this.props.curPage * 10 + e.index][this.props.primary];
        console.log('current page --- ', this.props.curPage, this.props.currentIndex)
        console.log('primary key --- ', this.props.primary, e.value[this.props.primary]);
        console.log('notes ------- ', this.props.notes);
        console.log('fields ------- ', this.props.fields);

        let pkInfo: FieldInfo;
        for (const f of this.props.fields) {
            if (f.name === this.props.primary) {
                pkInfo = f;
            }
        }

        console.log('pkInfo ----------- ', pkInfo);

        // 对于复合主键，需要单独处理
        if (pkInfo.ftype.type > 20) {
            encodeTableStruct(pkInfo.name, JSON.stringify(e.value[this.props.primary]), pkBin => {
                encodeTableStruct(this.props.currentTableName, JSON.stringify(e.value), encoded => {
                    writeDb(this.props.currentDatabaseName, this.props.currentTableName, pkBin, encoded, (r) => {
                        if (r) {
                            this.props.changeNum = -1;
                            popNewMessage('修改成功');
                            this.paint();
                            
                            this.getTableData(100);
            
                        } else {
                            popNewMessage('修改失败','error');
                        }
                    })
                })
            })
        } else {
            const bon = new BonBuffer();
            bon.write(e.value[this.props.primary]);

            encodeTableStruct(this.props.currentTableName, JSON.stringify(e.value), enocded => {
                writeDb(this.props.currentDatabaseName, this.props.currentTableName, bon.getBuffer(), enocded, (r) => {
                    if (r) {
                        this.props.changeNum = -1;
                        popNewMessage('修改成功');
                        this.paint();
                        
                        this.getTableData(100);
        
                    } else {
                        popNewMessage('修改失败','error');
                    }
                });
            });
        }
    }

    // 删除数据
    public delData(e:any) {
        if (this.props.addData) { // 删除新增
            this.props.changeNum = -1;
            this.props.addData = false;
            this.props.showDataList.splice(e.index,1);
            this.paint();

            return;
        }
        const key = this.props.showDataList[e.index][this.props.primary];// 主键值

        const bon = new BonBuffer();
        bon.write(key);

        const bon2 = new BonBuffer();
        bon2.write(undefined);

        writeDb(this.props.currentDatabaseName, this.props.currentTableName, bon.getBuffer(), bon2.getBuffer(), r => {
            if (r) {
                popNewMessage('删除成功');
                this.props.showDataList.splice(e.index,1);
                this.paint();
            } else {
                popNewMessage('删除失败','error');
            }
        });
    }

    // 新增一条数据
    public addData(e:any) {
        const bon = new BonBuffer();
        bon.write(e.value[this.props.primary]);

        console.log('sssssssssss ', JSON.stringify(e.value));

        encodeTableStruct(this.props.currentTableName, JSON.stringify(e.value), enocded => {
            writeDb(this.props.currentDatabaseName, this.props.currentTableName, bon.getBuffer(), enocded, (r) => {
                if (r) {
                    this.getTableData(100);
                    this.props.addData = false;
                    this.props.changeNum = -1;
                    popNewMessage('插入成功');
                    this.paint();
                } else {
                    popNewMessage('插入失败','error');
                }
            });
        });
    }

    // 添加按钮
    public addBtn() {
        const data = {};
        console.log('show data lsit ', this.props.showDataList[0]);
        console.log('show title list', this.props.showTitleList);
        for (const key in this.props.showDataList[0]) {
            data[key] = '';
        }
        if (!this.props.showDataList[0]) {
            for (const key in this.props.showTitleList) {
                data[key] = '';
            }
        }
        console.log('data ---- ', this.props.datas);
        this.props.showDataList.unshift(data);
        this.props.changeNum = 0;
        this.props.addData = true;
        this.paint();
    }
    
    // 切换下拉框展示
    public changeShow(e:any,ind:number) {
        this.props.showList = [false,false];
        this.props.showList[ind] = e.value;
        this.paint();
    }

    // 页面点击
    public pageClick() {
        this.props.showList = [false,false];
        this.paint();
    }
}