/**
 * 编辑管理员
 */
import { notify } from '../../../../pi/widget/event';
import { Widget } from '../../../../pi/widget/widget';
import { addUser, modifyAuth } from '../../net/rpc';
import { deepCopy } from '../../store/memstore';
import { popNewMessage } from '../../util/logic';
interface Props {
    powerList:string[]; // 所有权限列表
    selectList:number[]; // 选中列表
    name:string; // 用户名
    pwd:string; // 密码
    modify:boolean; // 仅编辑权限
}
export const PowerList = ['查看数据库','操作数据库','用户管理','查看日志','RPC调用','SHELL操作','TOPIC订阅'];  // 权限列表，对应于 AuthEnum

export class EditManager extends Widget {
    public props:Props = {
        powerList:PowerList,
        selectList:[0],
        name:'',
        pwd:'',
        modify:false
    };

    public setProps(props:any) {
        console.log(props);
        this.props = {
            ...this.props,
            ...deepCopy(props)
        };
        super.setProps(this.props);
    }

    public selected(index:number) {
        const ind = this.props.selectList.indexOf(index);
        if (ind === -1) {
            this.props.selectList.push(index);
        } else {
            this.props.selectList.splice(ind,1);
        }
        this.paint();
    }

    public nameChange(e:any) {
        this.props.name = e.value;
    }

    public pwdChange(e:any) {
        this.props.pwd = e.value;
    }

    public save(e:any) {
        if (this.props.modify) {
            modifyAuth(this.props.name,this.props.selectList,(res) => {
                console.log('设置用户权限',res);
                if (res && res.code === 1) {
                    popNewMessage('修改权限成功');
                    this.cancel(e,true);
                } else {
                    popNewMessage('修改权限失败','error');
                }
            });

            return;
        }

        if (!this.props.name || !this.props.pwd || this.props.selectList.length === 0) {
            popNewMessage('内容不能为空');
        }
        addUser(this.props.name,this.props.pwd,(r) => {
            console.log('添加用户',r);
            if (r && r.code === 1) {
                modifyAuth(this.props.name,this.props.selectList,(res) => {
                    console.log('设置用户权限',res);
                    if (res && res.code === 1) {
                        popNewMessage('添加用户成功');
                        this.cancel(e,true);
                    } else {
                        popNewMessage('添加用户失败','error');
                    }
                });
            } else {
                popNewMessage('添加用户失败','error');
            }
        });
    }

    public cancel(e:any,value:boolean= false) {
        notify(e.node,'ev-editMana-cancel',{ value:value });
    }
}