/**
 * rpc调用
 */
import { Forelet } from '../../../../pi/widget/forelet';
import { Widget } from '../../../../pi/widget/widget';
import { clientRpcFunc } from '../../net/init';
import { callArbitaryRpc, getAllRpcsPath, getHttpRpcs, structNameHash } from '../../net/rpc';
import { deepCopy, register } from '../../store/memstore';
import { popNewMessage } from '../../util/logic';
interface Props {
    txt:string;  // 输入的内容
    path:string; // 选中的rpc方法路径
    placeHolder:string;  // 默认提示内容
    rpcList:string[]; 
    funcList:string[];
    rpcFullPath: string[];
}
export const forelet = new Forelet();

export class RpcFunc extends Widget {
    public ok:() => void;
    public props:Props = {
        txt:'',
        path:'',
        placeHolder:`第一行：http rpc 请求url, 如 http://127.0.0.1 \n第二行：http rpc请求的方法，如 GET/POST\n第三行，http rpc 请求的参数，json字符串\n\n\n例子：\n\n http://127.0.0.1\n/login\n`,
        rpcList:[],
        funcList:[],
        rpcFullPath: [] 
    };

    public create() {
        super.create();
        getHttpRpcs((r) => {
            this.props.rpcFullPath = r;
            const s = new Set<string>();
            for (const rpc of r) {
                s.add(rpc.split('.')[0]);
            }
            this.props.rpcList = [...s];
            this.paint();
        });
    }

    // 关闭右侧输入框
    public close() {
        this.props.path = '';
        this.paint();
    }

    // 选择rpc
    public select(ind:number) {
        if (this.props.rpcList[ind] !== this.props.path) {
            this.props.path = this.props.rpcList[ind];
            const funcs = [];
            for (const rpc of this.props.rpcFullPath) {
                // console.log('rpc, path ', rpc, this.props.path)
                if (rpc.split('.')[0] === this.props.path) {
                    funcs.push(rpc.split('.')[1]);
                }
            }
            this.props.funcList = funcs;
        } else {
            this.props.path = '';
            this.props.funcList = [];
        }
        
        this.paint();
    }
    
    // 输入命令
    public commandInput(e:any) {
        this.props.txt = e.value;
    }

    // 执行rpc方法
    public exeCommand() {
        const req = this.props.txt.split('\n');
        const url = req[0];
        const method = req[1];
        let param;
        if (req[2]) {
            param = JSON.parse(req[2]);
        }

        if (method === 'GET') {
            fetch(url).then(resp => {
                resp.text().then(text => {
                    forelet.paint(text);
                }).catch(err => {
                    console.log(err);
                });
            }).catch(err => {
                console.log(err);
            });
        }

        if (method === 'POST') {
            fetch(url, {
                body: JSON.stringify(param),
                headers: {
                    'Content-Type': 'application/json'
                },
                method: 'POST',
                mode: 'no-cors'
            }).then(resp => {
                resp.text().then(text => {
                    forelet.paint(text);
                }).catch(err => {
                    console.log(err);
                });
            }).catch(err => {
                console.log(err);
            });
        }
    }
}
register(`rpcResultMap`,(r) => {
    forelet.paint(JSON.stringify(r));
});