/**
 * log
 */
import { Widget } from '../../../../pi/widget/widget';
import { listAllUser, showLog } from '../../net/rpc';
import { deepCopy } from '../../store/memstore';
import { dateToString, parseDate } from '../../util/logic';
interface Props {
    showDataList:any[];
    titles:string[];
    datas:any[];
    startTime:string;
    endTime:string; 
    pages:number;  // 总页数
    curPage:number; // 当前页码
    operation:string[]; // 操作类型
    userList:string[]; // 用户列表
    showList:boolean[];  // 下拉框显示列表
    curUser:string;  // 当前选择的用户
    curOpera:string;  // 当前选择的操作类型
}
const LIMITNUM = 10;// 每页显示数量
const Operation = ['其他','增加','删除','修改','查询'];  // 操作类型，对应于 DBOperation

export class LogHistory extends Widget {
    public props:Props = {
        showDataList:[],
        titles:['操作人','时间','操作','记录'],
        datas:[],
        startTime:'',
        endTime:'',
        pages:1,
        curPage: 0,
        operation: Operation,
        userList:[],
        showList:[false,false,false],
        curUser:'',
        curOpera:''
    };

    public create() {
        super.create();
        const today = dateToString(Date.now()); 
        // 默认时间为今天
        this.props.startTime = today;
        this.props.endTime = parseDate(today,1);
        listAllUser((users) => {
            console.log(users);
            this.props.userList = users.map(item => {
                return item.userName;
            });
            this.paint();
        });
    }

    public changeDate(e:any) {
        this.props.startTime = e.value[0];
        this.props.endTime = e.value[1];
    }

    public search() {
        if (this.props.startTime && this.props.endTime) {
            const st = Date.parse(this.props.startTime).toString();
            const ed = Date.parse(this.props.endTime).toString();
            showLog(st,ed,(r) => {
                console.log(st,ed,r);
                this.props.datas = r.map(item => {
                    return [
                        item.user,
                        dateToString(Number(item.time),1),
                        item.arg ? Operation[item.arg.op] :Operation[0],
                        `${item.module} ${JSON.stringify(item.arg)}`
                    ];
                });
                this.filterData(this.props.curUser,this.props.curOpera);
                this.paint();
            });
        }
    }

    // 过滤用户
    public changeUser(e:any) {
        if (e.index > -1) {
            this.props.curUser = e.value;
        } else {
            this.props.curUser = '';
        }
        this.filterData(this.props.curUser,this.props.curOpera);
        this.paint();
    }

    // 过滤操作类型
    public changeType(e:any) {
        if (e.index > -1) {
            this.props.curOpera = e.value;
        } else {
            this.props.curOpera = '';
        }
        this.filterData(this.props.curUser,this.props.curOpera);        
        this.paint();
    }

    /**
     * 过滤结果
     */
    public filterData(user:string,opera:string) {
        let datas = deepCopy(this.props.datas);
        if (user) {  // 过滤用户
            datas = datas.filter(item => {
                return item[0] === user;
            });
        }
        if (opera) {  // 过滤操作
            datas = datas.filter(item => {
                return item[2] === opera;
            });
        }
        this.props.showDataList = datas;
        this.props.pages = Math.floor(datas.length / LIMITNUM) + 1;
    }

    // 切换下拉框展示
    public changeShow(e:any,ind:number) {
        this.props.showList = [false,false,false];
        this.props.showList[ind] = e.value;
        this.paint();
    }

    // 页面点击
    public pageClick() {
        this.props.showList = [false,false,false];
        this.paint();
    }

    // 切换显示页
    public changeCurrent(e:any) {
        this.props.curPage = e.value;
        this.paint();
    }

}