/**
 * rpc调用
 */
import { Forelet } from '../../../../pi/widget/forelet';
import { Widget } from '../../../../pi/widget/widget';
import { getAllRpcsPath, callArbitaryRpc, structNameHash, decodeBinary } from '../../net/rpc';
import { deepCopy, register } from '../../store/memstore';
import { popNewMessage } from '../../util/logic';
import { clientRpcFunc } from '../../net/init';
interface Props {
    txt:string;  // 输入的内容
    path:string; // 选中的rpc方法路径
    placeHolder:string;  // 默认提示内容
    rpcList:string[]; 
    funcList:string[];
    rpcFullPath: string[];
}
export const forelet = new Forelet();

export class RpcFunc extends Widget {
    public ok:() => void;
    public props:Props = {
        txt:'',
        path:'',
        placeHolder:`第一行：rpc 名字\n第二行：rpc请求结构的名字\n第三行：rpc请求结构的json字符串\n例子：\n\npi_pt/util/platmgr.tabMeta\npi_pt/util/platmgr.TabMetaReq\n{"ware":"file","tab":"pipt_demo/database/testdb.TestTable10"}`,
        rpcList:[],
        funcList:[],
        rpcFullPath: [], 
    };

    public create() {
        super.create();
        getAllRpcsPath((r) => {
            this.props.rpcFullPath = r;
            let s = new Set<string>();
            for (const rpc of r) {
                s.add(rpc.split('.')[0]);
            }
            this.props.rpcList = [...s];
            this.paint();
        })
    }

    // 关闭右侧输入框
    public close() {
        this.props.path = '';
        this.paint();
    }

    // 选择rpc
    public select(ind:number) {
        if(this.props.rpcList[ind] !== this.props.path){
            this.props.path = this.props.rpcList[ind];
            let funcs = [];
            for (const rpc of this.props.rpcFullPath) {
                // console.log('rpc, path ', rpc, this.props.path)
                if (rpc.split('.')[0] === this.props.path) {
                    funcs.push(rpc.split('.')[1]);
                }
            }
            this.props.funcList = funcs;
        }else{
            this.props.path = '';
            this.props.funcList = [];
        }
        
        this.paint();
    }
    
    // 输入命令
    public commandInput(e:any) {
        this.props.txt = e.value;
    }

    // 执行rpc方法
    public exeCommand() {
        let req = this.props.txt.split('\n');
        if (req.length !== 4) {
            throw new Error("extactly 4 params");
        }
        const [rpcName, reqStructName, reqJstr, respStructName] = req;
        if (reqStructName && reqJstr) {
            callArbitaryRpc(rpcName, reqStructName, reqJstr, respStructName, r => {
                forelet.paint(JSON.stringify(r));
            });
            return;
        }

        if (!reqStructName && !reqJstr) {
            clientRpcFunc(rpcName, null, r => {
                if (r instanceof Uint8Array) {
                    decodeBinary(r, rr => {
                        forelet.paint(rr);
                    })
                } else {
                    forelet.paint(r);
                }
            });
            return;
        }

        if (reqJstr) {
            let param = reqJstr.split(':');
            if (param[0] === 'string') {
                clientRpcFunc(rpcName, param[1], r => {
                    forelet.paint(r);
                })
            } else if (param[0] === "boolean") {
                if (param[1] === "true") {
                    clientRpcFunc(rpcName, true, r => {
                        forelet.paint(r);
                    })
                } else if (param[1] === "false") {
                    clientRpcFunc(rpcName, false, r => {
                        forelet.paint(r);
                    })
                }
            } else if (param[0] === "number") {
                clientRpcFunc(rpcName, parseInt(param[1]), r => {
                    forelet.paint(r);
                })
            }
        }
        
        // if (req.length === 3) { // 请求参数是结构体
        //     callArbitaryRpc(req[0], req[1], req[2], r => {
        //         forelet.paint(r)
        //     })
        // } else if (req.length === 1) { // 没有请求参数
        //     clientRpcFunc(req[0], null, r => {
        //         forelet.paint(r);
        //     })
        // } else if (req.length === 2) { // 参数是基本类型
        //     let param = req[1].split(':');
        //     if (param[0] === "string") {
        //         clientRpcFunc(req[0], param[1], r => {
        //             forelet.paint(r);
        //         })
        //     } else if (param[0] === "boolean") {
        //         if (param[1] === "true") {
        //             clientRpcFunc(req[0], true, r => {
        //                 forelet.paint(r);
        //             })
        //         } else if (param[1] === "false") {
        //             clientRpcFunc(req[0], false, r => {
        //                 forelet.paint(r);
        //             })
        //         }
        //     } else if (param[0] === "number") {
        //         clientRpcFunc(req[0], parseInt(param[1]), r => {
        //             forelet.paint(r);
        //         })
        //     }
        // }
    }
}
register(`rpcResultMap`,(r) => {
    forelet.paint(JSON.stringify(r));
});