/**
 * topic
 */
import { popNew } from '../../../../pi/ui/root';
import { Widget } from '../../../../pi/widget/widget';
import { subDb, unSubDb } from '../../net/rpc';
import { getLocalStorage, setLocalStorage } from '../../store/lcStore';
import { dateToString, popNewMessage } from '../../util/logic';
interface Props {
    topicList:any[];  
    selectList:string[];  // 选中的主题名称
    allSelected:boolean;  // 全选
    topicResult:any[];  // 订阅主题返回值
    curTopic:string;  // 要查看的主题
    curResult:any[];  // 当前查看的主题的返回值
}

export class TopicSub extends Widget {
    public props:Props = {
        topicList:[
            // { topic:'111',path:'management/server/rpc/user.Login' }
        ],
        selectList:[],
        allSelected:false,
        topicResult:[
            // { title:'123',time:'14:02',msg:'111111111111' },
            // { title:'123',time:'14:02',msg:'111111111111' }
        ],
        curTopic:'',
        curResult:[]
    };

    public create() {
        super.create();
        this.props.topicList = getLocalStorage('topicList') || [];
    }

    // 选择某个主题
    public select(index:number) {
        const item = this.props.topicList[index];
        const ind = this.props.selectList.indexOf(item.topic);
        if (ind === -1) {
            this.props.selectList.push(item.topic);
            this.subTopic(item);
        } else {
            this.props.allSelected = false;
            this.props.selectList.splice(ind,1);
            unSubDb(item.topic);
        }
        this.paint();
    }

    // 全选所有主题
    public allSelect() {
        this.props.allSelected = !this.props.allSelected;
        if (this.props.allSelected) {
            for (let i = 0;i < this.props.topicList.length;i++) {
                const item = this.props.topicList[i];
                if (this.props.selectList.indexOf(item.topic) === -1) {
                    this.props.selectList.push(item.topic);
                    this.subTopic(item);
                }
            }

        } else {
            for (let i = 0;i < this.props.topicList.length;i++) {
                const name = this.props.topicList[i].topic;
                unSubDb(name);
            }
            this.props.selectList = [];
        }
        this.paint();
    }

    // 订阅主题
    public subTopic(item:any) {
        subDb(item.topic, item.path, (r) => {
            this.props.topicResult.unshift({
                title:item.topic,
                time:dateToString(Date.now(),2).split(' ')[1],
                msg:JSON.stringify(r)
            });
            if (this.props.curTopic === item.topic) {
                this.props.curResult.unshift(this.props.topicResult[0]);
            }
            this.paint();
        });
    }

    // 添加topic
    public addTopic() {
        popNew('management-client-components-modalBoxInput',{
            title:'添加topic',
            placeHolder:['请输入topic名称']
        },(r) => {
            if (r && r.topic) {
                this.props.topicList.push(r);
                this.select(this.props.topicList.length - 1);
                setLocalStorage('topicList',this.props.topicList);
                this.paint();
            } else {
                popNewMessage('输入内容不符合规则','error');
            }
        });
    }

    // 删除某个topic
    public deleteTopic(i:number) {
        const name = this.props.topicList[i].topic;
        unSubDb(name);
        this.props.topicList.splice(i,1);
        setLocalStorage('topicList',this.props.topicList);
        this.paint();
    }

    public showDetail(num:number) {
        this.props.curTopic = this.props.topicList[num].topic;
        this.props.curResult = this.props.topicResult.filter(item => {
            return item.title === this.props.curTopic;
        });
        this.paint();
    }

    public closeDetail() {
        this.props.curTopic = '';
        this.props.curResult = [];
        this.paint();
    }

}