// tslint:disable-next-line:missing-jsdoc
import { notify } from '../../../pi/widget/event';
import { Widget } from '../../../pi/widget/widget';
import { popNewMessage } from '../util/logic';
import { deepCopy } from '../store/memstore';

interface Props {
    title:any[];// 表格标题
    datas:any[];// 表数据
    selectList:any[];// 选择框列表
    allChecked:boolean;// 全选
    isChange:number;// 是否修改数据值
    isAdd:boolean;// 是否是新增数据
    needOperator:boolean; // 是否需要操作一栏
    pages:number; // 总页数
    curPage:number; // 当前页码
    orgRowData:any[] // 操作行的原始数据
    editRowData:any[];// 操作行的修改后数据
}
// tslint:disable-next-line:completed-docs
export class Table extends Widget {
    public props:Props = {
        title:[],
        datas:[],
        selectList:[],
        allChecked:false,
        isChange:-1,
        editRowData:[],
        isAdd:false,
        needOperator:true,
        pages:1,
        curPage:0,
        orgRowData:[]
    };
    
    public setProps(props:any) {
        this.props = {
            ...this.props,
            ...props
        };
        super.setProps(this.props);
        this.props.orgRowData = deepCopy(this.props.datas[0]);
        this.props.editRowData = deepCopy(this.props.datas[0]);
    }
    
    // 全选整页
    public allChecked() {
        this.props.allChecked = !this.props.allChecked;
        this.props.selectList = [];
        if (this.props.allChecked) {
            for (let i = 0;i < this.props.datas.length;i++) {
                this.props.selectList.push(i);
            }
        }
        this.paint();
    }

    // 选中对应的行
    public checked(index:number) {
        const ind = this.props.selectList.indexOf(index);
        if (ind === -1) {
            this.props.selectList.push(index);
        } else {
            this.props.allChecked = false;
            this.props.selectList.splice(ind,1);
        }
        this.paint();
    }

    // 行数据发生变化
    public rowDataChange(e:any,key:string) {
        this.props.editRowData[key] = e.value;
        console.log('原始数据',this.props.orgRowData[key],'修改数据',this.props.editRowData[key])
    }

    // 修改或保存数据
    public editAndSave(e:any,index:number) {
        // 修改数据
        if (this.props.isChange === -1) {
            this.props.isChange = index;
            this.props.editRowData = deepCopy(this.props.datas[index]);
            this.props.orgRowData = deepCopy(this.props.datas[index]);
            notify(e.node,'ev-editData-table',{ value:index });
            
        } else {
            const editData = this.props.editRowData;
            const orgData = this.props.orgRowData;
            for (const key in editData) {
                if (editData[key] !== orgData[key]) {
                    if(editData[key] === ''){
                        popNewMessage('输入不能为空','error');

                        return;
                    }else if (typeof orgData[key] === 'string') {
                        try {
                            editData[key] = JSON.parse(editData[key]);
                        } catch {

                        }
                    }
                } 
            }
            
            if (this.props.isAdd) {
                // 新增数据
                notify(e.node,'ev-addData-table',{ value:editData,index });
                this.props.isAdd = false;
                this.props.isChange = -1;
            
            } else  {
                // 保存修改
                notify(e.node,'ev-changeData-table',{ value:editData,index });
                this.props.isChange = -1;
            } 
        }
        this.paint();
    }
    
    // 删除数据
    public remove(e:any,index:number) {
        notify(e.node,'ev-deleteData-table',{ index });
    }
}
