import { addUser as aUser, changePasswd as cPasswd, delUser as dUser, getUserAuth as guAuth, listAllUser as lsuser, login, modifyAuth as mAuth } from '../../server/rpc/user.p';
import { AddUser, Auth, Login, Passwd } from '../../server/rpc/user.s';
import { ResultBin, ResultStr } from '../../utils/result.s';
import { clientRpcFunc } from './init';

import { arrayKey, read, select, write } from '../../server/rpc/db_mgr.p';
import { DBOperation, MetaInfo, Read, Select, Write } from '../../server/rpc/db_mgr.s';

import { showLog as sLog } from '../../server/rpc/log.p';
import { ShowLog, UserLog, WriteLog } from '../../server/rpc/log.s';

import { BonBuffer } from '../../../pi/util/bon';

import * as MetaInit from '../../../pi/struct/meta_init';

import { UserAuth } from '../../server/data/db/auth.s';
import { User } from '../../server/data/db/user.s';
import { subscribe, unSubscribe } from './init';

import { DecodeStructReq, DecodeStructResp, EncodeStructReq, IterDBReq, IterDBResp, TabMetaReq, WriteDBReq, WriteBackDataReq } from '../../server/rpc/platmgr.s';

declare var pi_modules;

MetaInit.init();
const structMgr = MetaInit.mgr;

export const readDb = (ware: string, tab: string, keys: any, cb: (r: ResultBin) => void) => {
    const bon = new BonBuffer();

    bon.writeArray(keys, el => {
        const b = new BonBuffer();
        (<any>el).bonEncode(b);
        bon.writeBin(b.getBuffer());
    });

    const read_info = new Read();
    read_info.tab = tab;
    read_info.keys = bon.getBuffer();
    read_info.ware = ware;
    read_info.op = DBOperation.QUERY;

    clientRpcFunc(read, read_info, (r: ResultBin) => {
        cb(r);
    });
};

// 检查是不是undefined序列化的
function equal (buf1, buf2) {
    if (buf1.byteLength !== buf2.byteLength) return false;
    const dv1 = new Uint8Array(buf1);
    const dv2 = new Uint8Array(buf2);
    for (let i = 0 ; i !== buf1.byteLength ; i++) {
        if (dv1[i] !== dv2[i]) return false;
    }

    return true;
}

(<any>self).dbRead = (ware: string, tab: string, keys: any) => {
    readDb(ware, tab, keys, (r) => {
        console.log('code: ', r.code);
        const [modName, expt] = tab.split('.');
        const t = pi_modules[modName + '.s'].exports[expt];

        const bon1 = new BonBuffer(r.msg);
        const values = bon1.readArray(() => {
            const bin = bon1.readBin();
            // undefined 序列化为 0
            if (equal(bin.buffer, new Uint8Array([0]).buffer)) {
                return null;
            }
            // 解析数据表
            const b = new BonBuffer(bin);
            const m = new t();
            m.bonDecode(b);

            return m;
        });

        console.log(values);
    });
};

export const iterDb = (ware: string, tab: string, pagination: number, startKey: any, endKey: any, dir: boolean = true, cb: (r) => void) => {
    const bon1 = new BonBuffer();
    bon1.write(startKey);

    const bon2 = new BonBuffer();
    bon2.write(endKey);

    const req = new IterDBReq();
    req.ware = ware;
    req.tab = tab;
    req.pagination = pagination;
    req.startKey = bon1.getBuffer();
    req.endKey = bon2.getBuffer();
    req.direction = dir;

    clientRpcFunc('pi_pt/util/platmgr.iterDB', req, (r) => {
        cb(r);
    });
};

// 迭代通过 .rs 文件生成的数据表的例子:

// 升序迭代所有记录： dbIter("file", "management/server/data/db/user.User", 10, undefined, undefined, true)
// 降序迭代所有记录：dbIter("file", "management/server/data/db/user.User", 10, undefined, undefined, false)
// 只能用来操作通过 .rs 文件生成的数据表， 因为要使用 pi_modules 里面的信息， 只有通过 .rs 文件生成的数据表才被注册到了 pi_modules
(<any>self).dbIter = (ware: string, tab: string, pagination: number, startKey: any, endKey: any, dir: boolean = true) => {
    iterDb(ware, tab, pagination, startKey, endKey, dir, r => {
        console.log('code', r);
        const [modName, expt] = tab.split('.');
        const t = pi_modules[modName + '.s'].exports[expt];

        if (r.code !== 1) {
            console.log('dbiter error');

            return;
        }

        const bon1 = new BonBuffer(r.msg);
        const values = bon1.readArray(() => {
            const bin = bon1.readBin();
            // undefined 序列化为 0
            if (equal(bin.buffer, new Uint8Array([0]).buffer)) {
                return null;
            }
            // 解析数据表
            const b = new BonBuffer(bin);
            const m = new t();
            m.bonDecode(b);

            return m;
        });

        console.log(values);
    });
};

export const writeDb = (ware: string, tab: string, key: Uint8Array, value: Uint8Array, cb: (r: ResultStr) => void) => {
    // const bon1 = new BonBuffer();
    // bon1.writeArray(keys, el => {
    //     const b = new BonBuffer();
    //     if (el !== undefined && el !== undefined) {
    //         (<any>el).bonEncode(b);
    //     } else {
    //         b.write(el);
    //     }
    //     bon1.writeBin(b.getBuffer());
    // });

    // const bon2 = new BonBuffer();
    // bon2.writeArray(values, el => {
    //     const b = new BonBuffer();
    //     if (el !== undefined && el !== undefined) {
    //         (<any>el).bonEncode(b);
    //     } else {
    //         b.write(el);
    //     }
    //     bon2.writeBin(b.getBuffer());
    // });

    // const writeInfo = new Write();
    // writeInfo.tab = tab;
    // writeInfo.key = bon1.getBuffer();
    // writeInfo.value = bon2.getBuffer();
    // writeInfo.ware = ware;
    // writeInfo.op = op;

    // clientRpcFunc(write, writeInfo, (r: ResultStr) => {
    //     cb(r)
    // });

    const req = new WriteDBReq();
    req.ware = ware;
    req.tab = tab;
    req.key = key;
    req.value = value;

    clientRpcFunc('pi_pt/util/platmgr.writeDB', req, r => {
        cb(r);
    });
}; 

(<any>self).dbWrite = (ware: string, tab: string, keys: any, values: any) => {
    writeDb(ware, tab, keys, values, r => {
        console.log('write result: ', r);
    });
};

export const _writeDB = (ware: string, tab: string, key: any, value: any, cb: (r) => void) => {
    const req = new WriteDBReq();
    req.ware = ware;
    req.tab = tab;

    const bon  = new BonBuffer();
    bon.write(key);
    req.key = bon.getBuffer();
    req.value = new Uint8Array(0);

    clientRpcFunc('pi_pt/util/platmgr.writeDB', req, r => {
        cb(r);
    });
};

(<any>self)._writeDB = (ware: string, tab: string, key: any, value: any) => {
    _writeDB(ware, tab, key, value, (r) => {
        console.log('r rr', r);
    });

    const bon = new BonBuffer();
    bon.write([1,2,2]);
    console.log('buffer ', bon.getBuffer());
};

(<any>self)._writeDB2 = (ware: string, tab: string, key: any, value: any) {
    
};

export const tables = (ware: string, cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/platmgr.getTablesForWare', ware, (r) => {
        cb(r.split(' '));
    });
};

(<any>self).getTabs = (ware: string) => {
    tables(ware, r => {
        console.log('get tabs: ', r);
    });
};

export const wares = (cb: (r) => void)  => {
    clientRpcFunc('pi_pt/util/platmgr.getWares', undefined, (r) => {
        cb(r.split(' '));
    });
};

(<any>self).allWares = () => {
    wares(r => {
        console.log('get all wares: ', r);
    });
};

export const metaInfo = (ware: string, tab: string, cb: (r) => void) => {
    const meta = new TabMetaReq();
    meta.ware = ware;
    meta.tab = tab;

    clientRpcFunc('pi_pt/util/platmgr.tabMeta', meta, (r) => {
        console.log('metainfo json ', JSON.parse(r));
        cb(r);
    });
};

(<any>self).getMetaInfo = (ware: string, tab: string) => {
    metaInfo(ware, tab, r => {
        console.log('getMetaInfo -----  ', r);
    })
};

// 导出所有rpc路径
export const getAllRpcsPath = (cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/platmgr.allRpcs', undefined, r => {
        cb(r.split(' '));
    });
};

(<any>self).getAllRpcsPath = () => {
    getAllRpcsPath(r => {
        console.log('all rpc paths --- ', r);
    });
};

// 导出某个rpc用到的rpc使用的结构
export const getRpcPathExports = (rpcPath: string) => {
    const exps = [];
    for (const e in pi_modules[rpcPath].exports) {
        exps.push(e);
    }

    return exps;
};

(<any>self).getRpcPathExports = (rpcPath: string) => {
    return getRpcPathExports(rpcPath);
};

// 订阅数据库
export const subDb = (path: string, tabName: string, cb: Function) => {
    // const [modName, expt] = tabName.split('.');
    // const t = pi_modules[modName + '.s'].exports[expt];

    // subscribe(path, t, cb);
    subscribe(path, undefined, cb);
};

(<any>self).subDb = (path: string, tabName: string) => {
    subDb(path, tabName , r => {
        console.log(r);
    });
};

export const unSubDb = (path: string) => {
    console.log('unsub -- ', path);
    unSubscribe(path);
};

(<any>self).unsubDb = (path: string) => {
    unSubDb(path);
};

export const encodeTableStruct = (structName: string, jstr: string, cb: (r) => void) => {
    const req = new EncodeStructReq();
    req.structName = structName;
    req.jstr = jstr;

    clientRpcFunc('pi_pt/util/platmgr.encodeStruct', req, r => {
        cb(r);
    });
};

export const getHttpRpcs = (cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/platmgr.getAllHttpRpcEndpoint', undefined, r => {
        cb(r.split(' '));
    });
};

export const getMigrateDataTables = (cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/migration.migrateDataTables', undefined, r => {
        cb(r.split(' '));
    });
}

export const migrationTable = (tabName: string, cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/migration.migrationTable', tabName, r => {
        cb(r.split(' '));
    });
}

export const deleteTempDataTable = (tabName: string, cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/migration.deleteTempTableData', tabName, r => {
        cb(r)
    });
}

export const writeBackData = (oldTabName: string, tmpTabName: string, cb: (r) => void) => {
    let req = new WriteBackDataReq();
    req.oldTabName = oldTabName;
    req.tempTabName = tmpTabName;

    encodeTableStruct(WriteBackDataReq._$info.name, JSON.stringify(req), bin => {
        clientRpcFunc('pi_pt/util/migration.writeBackData', bin, r => {
            cb(r);
        })
    });
}

export const decodeTableStruct = (structName: string, bin: Uint8Array[], cb: (r: DecodeStructResp) => void) => {
    const req = new DecodeStructReq();
    req.structName = structName;
    req.bin = bin;

    clientRpcFunc('pi_pt/util/platmgr.decodeStruct', req, r => {
        cb(r);
    });
};

export const decodeBinary = (bin: Uint8Array, cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/platmgr.decodeBinary', bin, r => {
        cb(r);
    });
}

// 根据结构体的名字获取hash
export const structNameHash = (structName: string, cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/platmgr.getStructNameHash', structName, r => {
        cb(r);
    });
};

(<any>self).structNameHash = (structName: string) => {
    structNameHash(structName, r => {
        console.log('structNameHash ----  ', r);
    })
}

// 调用任意项目的rpc
export const callArbitaryRpc = (rpcName: string, reqStructName: string, jstr: string, respStructName: string, cb: (r) => void) => {
    if (jstr == null) {
        jstr = "";
    }
    encodeTableStruct(reqStructName, jstr, rr => {
        clientRpcFunc(rpcName, rr, rrr => {
            if (rrr instanceof Uint8Array) {
                decodeTableStruct(respStructName, [rrr], r => {
                    cb(r);
                })
            } else {
                cb(rrr);
            }
        });
    });
};
(<any>self).callArbitaryRpc = callArbitaryRpc;

// ====================================================================
// ====================================================================

// 用户登录
export const loginUser = (userName: string, pwd: string, cb: (r: ResultStr) => void) => {
    const log = new Login();
    log.user = userName;
    log.pwd = pwd;

    clientRpcFunc(login, log, (r: ResultStr) => {
        cb(r);
    });
};

(<any>self).login = (userName: string, pwd: string) => {
    loginUser(userName, pwd, (r) => {
        console.log(r);
    });
};

// 添加用户
export const addUser = (userName: string, pwd: string, cb: (r: ResultStr) => void) => {
    const u = new User;
    u.user = userName;
    u.pwd = pwd;

    clientRpcFunc(aUser, u, (r: ResultStr) => {
        cb(r);
    });
};

(<any>self).addUser = (userName: string, pwd: string) => {
    addUser(userName, pwd, r => {
        console.log(r);
    });
};

// 删除用户
export const delUser = (userName: string, cb: (r: ResultStr) => void) => {
    clientRpcFunc(dUser, userName, r => {
        cb(r);
    });
};

(<any>self).delUser = (userName: string) => {
    delUser(userName, r => {
        console.log(r);
    });
};

// 修改用户授权信息
export const modifyAuth = (userName: string, auth: number[], cb: (r: ResultStr) => void) => {
    const au = new Auth;
    au.user = userName;
    au.authList = auth;

    clientRpcFunc(mAuth, au, r => {
        cb(r);
    });
};

(<any>self).modifyAuth = (userName: string, auth: number[]) => {
    modifyAuth(userName, auth, r => {
        console.log(r);
    });
};

export const changePasswd = (oldPass: string, newPass: string, cb: (r: ResultStr) => void) => {
    const p = new Passwd;
    p.old = oldPass;
    p.new = newPass;

    clientRpcFunc(cPasswd, p, r => {
        cb(r);
    });
};

(<any>self).changePasswd = (oldPass: string, newPass: string) => {
    changePasswd(oldPass, newPass, r => {
        console.log(r);
    });
};

export const listAllUser = (cb: (r) => void) => {
    clientRpcFunc(lsuser, null, r => {
        const bon = new BonBuffer(r.msg);
        const users = bon.readArray(() => {
            const bin = bon.readBin();
            const b = new BonBuffer(bin);
            const t = new UserAuth;
            t.bonDecode(b);

            return t;
        });
        cb(users);
    });
};

(<any>self).listAllUser = () => {
    listAllUser(r => {
        console.log('users: ', r);
    });
};

export const getUserAuth = (cb: (r) => void) => {
    clientRpcFunc(guAuth, undefined, r => {
        cb(r);
    });
};

(<any>self).getUserAuth = () => {
    getUserAuth(r => {
        console.log(r);
    });
};

// 查看某一时间段内的日志， 时间戳需要转换为字符串
export const showLog = (startTime: string, endTime: string, cb: (r: any) => void) => {
    if (!startTime || !endTime) {
        throw new Error('must specify startTime and endTime');
    }

    const log = new ShowLog;
    log.startTime = startTime;
    log.endTime = endTime;

    clientRpcFunc(sLog, log, r => {
        if (r.code !== 1) {
            return;
        }

        const bon = new BonBuffer(r.msg);
        const values = bon.readArray(() => {
            const bin = bon.readBin();
            // undefined 序列化为 0
            if (equal(bin.buffer, new Uint8Array([0]).buffer)) {
                return null;
            }
            // 解析数据表
            const b = new BonBuffer(bin);
            const m = new WriteLog;
            m.bonDecode(b);

            return m;
        });

        for (const v of values) {
            if (v.arg.length > 0) {
                const m = v.module.split(':')[2];
                switch (m) {
                    case 'Read': {
                        const b = new BonBuffer(v.arg);
                        const r = new Read();
                        r.bonDecode(b);
                        v.arg = <any>r;
                        const keyBon = new BonBuffer(r.keys);
                        const keys = keyBon.readArray(() => {
                            return keyBon.read();
                        });

                        const parsed_key = [];
                        for (let k of keys) {
                            const b = new BonBuffer(k);
                            k = b.read(); 
                            parsed_key.push(k);
                        }
                        r.keys = <any>parsed_key;
                        break;
                    }
                    case 'Select': {
                        const b = new BonBuffer(v.arg);
                        const sel = new Select();
                        sel.bonDecode(b);
                        const startKeyBon = new BonBuffer(sel.startKey);
                        const endKeyBon = new BonBuffer(sel.endKey);
                        const startKey = startKeyBon.read();
                        const endKey = endKeyBon.read();
                        sel.startKey = <any>startKey;
                        sel.endKey = <any>endKey;
                        v.arg = <any>sel;
                        break;
                    }
                    case 'Write': {
                        const b = new BonBuffer(v.arg);
                        const w = new Write();
                        w.bonDecode(b);
                        v.arg = <any>w;

                        if (w.key[0] !== 0) {
                            const keyBon = new BonBuffer(w.key);
                            const keys = keyBon.readArray(() => {
                                return keyBon.read();
                            });

                            const parsed_key = [];
                            for (let k of keys) {
                                const b = new BonBuffer(k);
                                k = b.read();
                                parsed_key.push(k);
                            }
                            w.key = <any>parsed_key;

                        } else {
                            w.key = <any>'';
                        }

                        if (w.value[0] !== 0) {
                            const valBon = new BonBuffer(w.value);
                            const values = valBon.readArray(() => {
                                const [modName, tabName] = w.tab.split('.');
                                const t  = pi_modules[modName + '.s'].exports[tabName];

                                const bin = valBon.readBin();
                                // undefined 序列化为 0
                                if (equal(bin.buffer, new Uint8Array([0]).buffer)) {
                                    return null;
                                }
                                // 解析数据表
                                const b = new BonBuffer(bin);
                                const m = new t();
                                m.bonDecode(b);

                                return m;
                            });
                            w.value = <any>values;
                        } else {
                            w.value = <any>values;
                        }
                        break;
                    }
                    case 'Auth': {
                        const b = new BonBuffer(v.arg);
                        const a = new Auth;
                        a.bonDecode(b);
                        v.arg = <any>a;

                        break;
                    }
                    case 'AddUser': {
                        const b = new BonBuffer(v.arg);
                        const add = new AddUser;
                        add.bonDecode(b);
                        v.arg = <any>add;
                    }
                    case 'Passwd': {
                        v.arg = null;
                        break;
                    }
                    case 'Login': {
                        const b = new BonBuffer(v.arg);
                        const lg = b.read();
                        v.arg = <any>lg;
                        break;
                    }
                    case 'string': {
                        const b = new BonBuffer(v.arg);
                        const arg = b.read();
                        v.arg = arg;
                    }
                }
            } else {
                v.arg = null;
            }
        }
        cb(values);
    });
};

(<any>self).showLog = (startTime: string, endTime: string) => {
    showLog(startTime, endTime, r => {
        console.log(r);
    });
};

// ===================== shell =======================
export const execShell = (cmd: string, cb: (r) => void) => {
    clientRpcFunc('pi_pt/util/platmgr.execShellCmd', cmd, r => {
        cb(r);
    });
};

(<any>self).execShell = (cmd: string) => {
    execShell(cmd, r => {
        console.log(r);
    });
};

export const testCallArrayKey = (cb: (r) => void): void => {
    clientRpcFunc(arrayKey, undefined, r => {
        cb(r);
    });
};

(<any>self).testCallArrayKey = () => {
    testCallArrayKey(r => {
        console.log(r);
    });
};