import { TabMeta } from '../../../pi/struct/sinfo';
import { write as bonWrite } from '../../../pi/struct/util';
import { BonBuffer } from '../../../pi/util/bon';
import { getTabmetaBuffer, iterDb, listAllTables, tabkvGetValue} from "../../../pi_pt/rust/pi_serv/js_db";
import { ResultBin, ResultStr } from '../../utils/result.s';
import { MetaInfo, Read, Select, Write } from './db_mgr.s';

import {Tr} from "../../../pi/db/mgr";
import { Tr as RawTr } from '../../../pi_pt/db/mgr';

import {Env} from "../../../pi/lang/env";

import { Session } from '../../../pi_pt/net/session';
import { Vec } from '../../../pi_pt/rust/def/vec';
import { arcDerefVec } from '../../../pi_pt/rust/pi_serv/js_base';
import { getAllWares as allWares, tabkvNew, tabkvWithValue } from '../../../pi_pt/rust/pi_serv/js_db';
import { bonbufCmp } from '../../../pi_pt/rust/pi_serv/js_lib';
import { AuthEnum } from '../data/db/auth.s';
import { kkkk } from '../data/db/foo.s';
import { writeLog } from './log.r';
import { WriteLog } from './log.s';
import { checkAuth, getUserName } from './user.r';

declare var env: Env;
/**
 * 数据库管理
 */

/**
 * 获取表数据 
 */
// #[rpc=rpcServer]
export const read = (table: Read): ResultBin => {
    const res = new ResultBin();

    // if (!checkAuth(AuthEnum.DBRO) || !checkAuth(AuthEnum.DBRW)) {
    //     const msg = new BonBuffer();
    //     msg.write("read auth failed");
    //     res.code = 3;
    //     res.msg = msg.getBuffer();
    //     return res;
    // }

    const bf = new BonBuffer();
    table.bonEncode(bf);
    const log = new WriteLog();
    log.user = getUserName();
    log.arg = bf.getBuffer();
    log.ip = env.get('ip');
    log.module = 'management/server/rpc/db_mgr.s:read:Read';
    log.action = "read db"

    writeLog(log);

    const dbMgr = env.dbMgr;

    if (!tableExist(table.ware, table.tab)) {
        const msg = new BonBuffer();
        bonWrite(`"${table.tab}": not found in ware: "${table.ware}"`, msg);
        res.code = 2; // 2表示这个表不存在
        res.msg = msg.getBuffer();

        return res;
    }

    // 解析key
    const bon = new BonBuffer(table.keys);
    const keys = bon.readArray(() => {
        return bon.readBin();
    });

    console.log("keys: ", keys)

    const queries = Vec.newTabKV();
    let values;

    if (table.ware === "memory") {
        for (let i = 0; i < keys.length; i++) {
            const kv = tabkvNew("memory", table.tab, keys[i]);
            queries.pushTabKV(kv)
        }
        dbMgr.read((tr: Tr) => {
            values = (<any>tr).inner.query(queries, 1000, false);
        });
        
    } else if (table.ware === "file") {
        for (let i = 0; i < keys.length; i++) {
            const kv = tabkvNew("file", table.tab, keys[i]);
            queries.pushTabKV(kv)
        }
        dbMgr.read((tr: Tr) => {
            values = (<any>tr).inner.query(queries, 1000, false);
        });
    }

    const rr = [];
    const valsTabKv = values.asSliceTabKV();
    for (let i = 0; i < valsTabKv.length; i++) {
        const arcVecU8 = tabkvGetValue(valsTabKv[i]);
        if (arcVecU8 === undefined || arcVecU8 === null) {
            const b = new BonBuffer();
            // 未查到数据返回undefined， 需要单独序列化
            b.write(undefined);
            rr.push(b.getBuffer());
        } else {
            const vecU8 = arcDerefVec(arcVecU8);
            const vec = vecU8.asSliceU8();
            rr.push(vec);
        }
    }

    const bonRes = new BonBuffer();

    bonRes.writeArray(rr, el => {
        bonRes.writeBin(el);
    });

    res.code = 1;
    res.msg = bonRes.getBuffer();

    return res;
};

/**
 * 遍历表数据
 */
// #[rpc=rpcServer]
export const select = (range: Select): ResultBin => {
    const res = new ResultBin();

    // if (!checkAuth(AuthEnum.DBRO) || !checkAuth(AuthEnum.DBRW)) {
    //     const msg = new BonBuffer();
    //     msg.write("select auth failed");
    //     res.code = 3;
    //     res.msg = msg.getBuffer();
    //     return res;
    // }

    const bf = new BonBuffer();
    range.bonEncode(bf);
    const log = new WriteLog();
    log.user = getUserName();
    log.arg = bf.getBuffer()
    log.ip = env.get('ip');
    log.module = 'management/server/rpc/db_mgr.s:select:Select';
    log.action = 'db range iteration';

    writeLog(log);

    const dbMgr = env.dbMgr;

    const startKeyBon = new BonBuffer(range.startKey);
    const isStartKeyUndefined = range.startKey[0] === 0;
    const isEndKeyUndefined = range.endKey[0] === 0;

    if (!tableExist(range.ware, range.tab)) {
        const msg = new BonBuffer();
        bonWrite(`"${range.tab}": not found`, msg);
        res.code = 2; // 2表示这个表不存在
        res.msg = msg.getBuffer();
        return res;
    }

    let dbType;
    if (range.ware === 'memory') {
        dbType = 'memory';
    } else if (range.ware === 'file') {
        dbType = 'file';
    }

    let pagination = range.pagination;
    const value = [];
    if (range.direction) {
        dbMgr.read((tr: Tr) => {
            let di;
            // 检查是否是 undefined
            if (equal(startKeyBon.getBuffer(), new Uint8Array([0]).buffer)) {
                di = iterDb((<any>tr).inner, dbType, range.tab, undefined, true, '');
            } else {
                di = iterDb((<any>tr).inner, dbType, range.tab, startKeyBon.getBuffer(), true, '');
            }
            while(pagination) {
                const elem = di.next();
                if (!elem) {
                    break;
                } else {
                    // console.log("elem1, endkey1: ", elem[0], endKey)
                    const k = arcDerefVec(elem[0]).asSliceU8();
                    const ordering = bonbufCmp(k, range.endKey);
                    const v = arcDerefVec(elem[1]).asSliceU8();

                    if (isStartKeyUndefined && !isEndKeyUndefined) {
                        if (ordering > 0) {
                            break;
                        } else {
                            value.push(v);
                        }
                    } else if (isStartKeyUndefined && isEndKeyUndefined ) {
                        value.push(v);
                    } else if (!isStartKeyUndefined && isEndKeyUndefined) {
                        value.push(v);
                    } else {
                        if (ordering > 0) {
                            break;
                        } else {
                            value.push(v);
                        }
                    }
                }
                pagination--;
            }
        });
        // console.log("value: ", value);
    } else {
        dbMgr.read((tr: Tr) => {
            let di;
            // 检查是否是 undefined
            if (equal(startKeyBon.getBuffer(), new Uint8Array([0]).buffer)) {
                di = iterDb((<any>tr).inner, dbType, range.tab, undefined, false, '');
            } else {
                di = iterDb((<any>tr).inner, dbType, range.tab, startKeyBon.getBuffer(), false, '');
            }
            while(pagination) {
                const elem = di.next();
                if (!elem) {
                    break;
                } else {
                    // console.log("elem1, endkey1: ", elem[0], endKey)
                    const k = arcDerefVec(elem[0]).asSliceU8();
                    const ordering = bonbufCmp(k, range.endKey);
                    const v = arcDerefVec(elem[1]).asSliceU8();

                    if (isStartKeyUndefined && !isEndKeyUndefined) {
                        if (ordering < 0) {
                            break;
                        } else {
                            value.push(v);
                        }
                    } else if (isStartKeyUndefined && isEndKeyUndefined ) {
                        value.push(v);
                    } else if (!isStartKeyUndefined && isEndKeyUndefined) {
                        value.push(v);
                    } else {
                        if (ordering < 0) {
                            break;
                        } else {
                            value.push(v);
                        }
                    }
                }
                pagination--;
            }
        });
        // console.log("value: ", value);
    }

    // bonWrite(value, msg);
    const msg = new BonBuffer();
    msg.writeArray(value, el => {
        msg.writeBin(el);
    });
    res.code = 1;
    res.msg = msg.getBuffer();

    return res;
};

/**
 * 插入/修改/删除表数据
 */
// #[rpc=rpcServer]
export const write = (table: Write): ResultStr => {
    const dbMgr = env.dbMgr;
    const r = new ResultStr();

    // if (!checkAuth(AuthEnum.DBRW)) {
    //     r.code = 3;
    //     r.msg = 'write auth failed';

    //     return r
    // }

    const bf = new BonBuffer();
    table.bonEncode(bf);
    const log = new WriteLog();
    log.user = getUserName();
    log.arg = bf.getBuffer();
    log.ip = env.get('ip');
    log.module = 'management/server/rpc/db_mgr.s:write:Write';

    const keysBon = new BonBuffer(table.key);
    const valuesBon = new BonBuffer(table.value);

    const keysArr = keysBon.readArray(() => {
        return keysBon.readBin();
    });

    const valuesArr = valuesBon.readArray(() => {
        return valuesBon.readBin();
    });

    if (!tableExist(table.ware, table.tab)) {
        r.code = 2;
        r.msg = `table ${table.tab} not found in ware: "${table.ware}"`;
        log.action = 'write data to db failed: table not found';
        writeLog(log);
        
        return r;
    }

    const modifies = Vec.newTabKV();
    for (let i = 0; i < keysArr.length; i++) {
        if (equal(valuesArr[i].buffer, new Uint8Array([0]).buffer)) {
            const kv = tabkvNew(table.ware, table.tab, keysArr[i]);
            modifies.pushTabKV(kv);
        } else {
            const kv = tabkvWithValue(table.ware, table.tab, keysArr[i], valuesArr[i]);
            modifies.pushTabKV(kv);
        }
    }

    try {
        dbMgr.write((tr: Tr) => {
            (<any>tr).inner.modify(modifies, 1000, false);
        });
        r.code = 1;
        r.msg = 'write success';
        log.action = 'write data to db success';
    } catch(e) {
        r.code = 2;
        r.msg = 'write failed';
        log.action = 'write data to db failed: database write error';
    }

    writeLog(log);

    return r;
};

// #[rpc=rpcServer]
export const getAllWares = (): ResultBin => {
    console.log('getAllWares --- ');
    const res = new ResultBin();
    
    // if (!checkAuth(AuthEnum.DBRO)) {
    //     const msg = new BonBuffer();
    //     msg.write('getAllWares auth failed');
    //     res.code = 3;
    //     res.msg = msg.getBuffer();
    //     return res;
    // }

    const wares = allWares((<any>env.dbMgr).inner).asSliceString();
    console.log('wares - ---',  wares)
    const msg = new BonBuffer();
    bonWrite(wares, msg)

    res.code = 1;
    res.msg = msg.getBuffer();

    return res;
}

// #[rpc=rpcServer]
export const getAllTables = (ware: string): ResultBin => {
    const dbMgr = env.dbMgr;
    const res = new ResultBin();

    // if (!checkAuth(AuthEnum.DBRO)) { 
    //     const msg = new BonBuffer();
    //     msg.write('getAllTables auth failed');
    //     res.code = 3;
    //     res.msg = msg.getBuffer();
    //     return res;
    // }

    let tables;
    dbMgr.read((tr: Tr) => {
        tables = listAllTables((<any>tr).inner, ware).asSliceString();
    });

    const msg = new BonBuffer();
    bonWrite(tables, msg);

    res.code = 1;
    res.msg = msg.getBuffer();

    return res;
};

// #[rpc=rpcServer]
export const getTabMetaInfo = (meta: MetaInfo): ResultBin => {
    console.log('meta ----- ', meta.tab, meta.ware)
    console.log('getTabMetaInfo ------')
    const res = new ResultBin();
    const info = extractTabMetaInfo(meta.ware, meta.tab);
    // console.log("info", info)

    // if (!checkAuth(AuthEnum.DBRO)) {
    //     const msg = new BonBuffer();
    //     msg.write('getTabMetaInfo auth failed');
    //     res.code = 3;
    //     res.msg = msg.getBuffer();
    //     return res;
    // }

    const msg = new BonBuffer();

    if(info.k === undefined || info.v === undefined) {
        res.code = 2; // tab doesn't exist
        
        return res;
    }

    // bonWrite(info, msg);
    info.bonEncode(msg);
    res.code = 1;
    res.msg = msg.getBuffer();

    // console.log("Msg len: ", res.msg.length);

    return res;
};

// #[rpc=rpcServer]
export const arrayKey =(): void => {
    const kv = new kkkk();
    kv.key = [1,2];
    // let bon = new BonBuffer();

    // bon.writeArray(kv.key, el => {
    //     let b = new BonBuffer();
    //     (<any>el).bonEncode(b);
    //     bon.writeBin(b.getBuffer());
    // });

    // console.log('xxxx ', bon.getBuffer())

    env.dbMgr.write((tr: Tr) => {
        tr.modify([
            {ware: 'file', tab: kkkk._$info.name, key: kv.key, value: kv}
        ], 1000, false)
    });

    env.dbMgr.read((tr: Tr) => {
        const q = tr.query([
            {ware: 'file', tab: kkkk._$info.name, key: kv.key}
        ], 1000, false);

        console.log('queried: ', q);
    });
};

const tableExist = (ware: string, tab: string): boolean => {
    const dbMgr = env.dbMgr;
    let tabs;

    dbMgr.read((tr: Tr) => {
        tabs = listAllTables((<any>tr).inner, ware).asSliceString();
    });

    if (tabs.indexOf(tab) < 0) {
        return false;
    }

    // console.log("tabs: ", tabs)

    return true;
};

// 取得表元信息
export const extractTabMetaInfo = (ware: string, tab: string): TabMeta => {
    const tabMeta = (<any>env.dbMgr).inner.tabInfo(ware, tab);

    const tm = new TabMeta();
    
    // console.log("tabMeta: ", tabMeta)

    if (tabMeta) {
        const buf = getTabmetaBuffer(tabMeta);
        const bon = new BonBuffer(buf.asSliceU8());
        tm.bonDecode(bon);
    }

    // console.log("tm: ", tm)

    return tm;
};

// 检查是不是undefined序列化的
function equal (buf1, buf2) {
    if (buf1.byteLength !== buf2.byteLength) return false;
    const dv1 = new Uint8Array(buf1);
    const dv2 = new Uint8Array(buf2);
    for (let i = 0 ; i !== buf1.byteLength ; i++) {
        if (dv1[i] !== dv2[i]) return false;
    }
    return true;
}
