/**
 * RPC， 远程方法调用
 * 采用 mqtt上定义的每会话的$req和$resp主题，来发送请求和接受响应
 */

// ============================== 导入
import { Func, Struct, StructMgr, structMgr } from '../struct/struct_mgr';
import { read, writeBon, writeBon2} from '../struct/util';
import { BonBuffer, SerializeType } from '../util/bon';
import { Client } from './mqtt_c';

/**
 * 创建一个RPC函数
 * @example
 */
export const create = (client: Client, mgr: StructMgr) : Rpc => {
    const mqttRpc = new MqttRpc(client, mgr);
    client.onMessage((topic, payload: Uint8Array) => {
        if(topic === '$r') {
            const bb = new BonBuffer(payload, 0, payload.length);
            const rid = bb.readU32();// 消息开始表示此次请求的id
            const timeout = bb.readU8();
            if(mqttRpc.wait[rid]) {
                try {
                    const x = read(bb, mgr);
                    mqttRpc.wait[rid](x);
                } catch {
                    // 不能通过 mgr 解析就返回二进制数据
                    const bb = new BonBuffer(payload, 0, payload.length);
                    const rid = bb.readU32();// 消息开始表示此次请求的id
                    const timeout = bb.readU8();
                    mqttRpc.wait[rid](bb.getBuffer());
                }
                delete mqttRpc.wait[rid];
            }
        }
    });
    return (name: string, req: SerializeType, callback:Func, timeout: number) => {
        if (req instanceof Uint8Array) {
            mqttRpc.call_bin(name, req, callback, timeout);
        } else {
            mqttRpc.call(name, req, callback, timeout);
        }
    };
};

class MqttRpc {
    public rid = 1;
    public wait = {};
    public client: Client;
    public mgr: StructMgr;
    constructor(client: Client, mgr: StructMgr) {
        this.client = client;
        this.mgr = mgr;
    }

    // 远程调用
    public call(name: string, req: SerializeType, callback: Func, timeout: number) {
        const bb = new BonBuffer();
        this.wait[this.rid] = callback;
        bb.writeU32(this.rid++);
        bb.writeU8(timeout);
        this.rid >= 0xffffffff && (this.rid = 1);
        if (req === null || req === undefined) {
            bb.writeNil();
        } else if (req instanceof Struct) {
            writeBon(req, bb);
        } else {
            req.bonEncode(bb);
        }
        this.client.publish(name, bb.getBuffer(),0,true);
    }

    public call_bin(name: string, structBin: Uint8Array, callback: Func, timeout: number) {
        console.log('struct bin ---- ', structBin);
        const bb = new BonBuffer();
        this.wait[this.rid] = callback;
        bb.writeU32(this.rid++);
        bb.writeU8(timeout);
        writeBon2(structBin, bb);

        console.log('call_bin bb --- ', bb.getBuffer());

        this.client.publish(name, bb.getBuffer(), 0, true);
    }
}

export type Rpc = (name: string, req: SerializeType, callback:Func, timeout: number) => void;
