/// <reference path="./babylon.d.ts"/>

import { ResTab, RES_TYPE_FILE, Res, loadOK, register } from "../../util/res_mgr";
import { utf8Decode } from "../../util/util";

//实现babylon的脱机提供者， 从外部ResTab获取数据
export class OfflineProvider implements BABYLON.IOfflineProvider {
    enableSceneOffline: boolean = true;
    enableTexturesOffline: boolean = true;
    resTab: ResTab = new ResTab();

    constructor(resTab: ResTab){
        this.resTab = resTab;
    }

    open(successCallback: () => void, _errorCallback: () => void) {
        successCallback();
    }
    
    loadImage(url: string, image: HTMLImageElement) {
        image.src = url;
    }

    loadFile(url: string, sceneLoaded: (data: any) => void, _progressCallBack?: (data: any) => void, errorCallback?: () => void, _useArrayBuffer?: boolean) {
        this.resTab.load(RES_TYPE_FILE + ":" + url, RES_TYPE_FILE, url, null, (r) => {
            if(url.endsWith(".gltf")){
                sceneLoaded(utf8Decode(r.link));
            }else{
                sceneLoaded(r.link);
            }
        }, errorCallback);
    }

    loadAnimationKeys(name: string): Array<BABYLON.IAnimationKey>{
        let r = this.resTab.get(anmationKey(name));
        if (r){
            return r.link;
        }else {
            return null;
        }
    }

    createAnimationKeys(name: string, keys: Array<BABYLON.IAnimationKey>){
        this.resTab.load(anmationKey(name), BABYLON_ANIMATION_TYPE, keys, null, null, () => {
            console.log("createAnimationKeys fail!, name:", name);
        });
    }

    release(){
        this.resTab.release();
    }
}

/**
 * @description 纹理资源
 * @example
 */
export class AnimationKeysRes extends Res {
    link: Array<BABYLON.IAnimationKey>;
	/**
	 * @description 创建
	 * @example
	 */
	public create(data: Array<BABYLON.IAnimationKey>): void {
		this.link = data;
	}
}

const BABYLON_ANIMATION_TYPE = "babyblon_animationkeys";
const anmationKey = (name: string): string => {
    return BABYLON_ANIMATION_TYPE + ":" + name;
}

const createGeometryRes = (name: string, type: string, keys: Array<BABYLON.IAnimationKey>, _: any) => {
	return loadOK(name, type, null, AnimationKeysRes, keys);
};

register(BABYLON_ANIMATION_TYPE, (name: string, type: string, keys: Array<BABYLON.IAnimationKey>, _: any) => {
	createGeometryRes(name, type, keys, _);
});

