
import {Tr, Mgr} from "./rust/pi_db/mgr"
import * as JsDb from "./rust/pi_serv/js_db"
import {BonBuffer} from "../pi/util/bon"
import {TabMeta} from "../pi/struct/sinfo"
import { dropNativeObj } from "./rust/pi_serv/js_base";

export interface Handler{
    (tr: Tr): any
}

/**
 * 读数据
 * @param timeout 超时时间， 默认10毫秒， 超时将抛出异常
*/
export const read = (mgr: Mgr, txhd: Handler, timeout = 10) => {
    let time = new Date().getTime();
    while(timeout > 0){
        let tr = mgr.transaction(false);
        let r;
        try {
            r = txhd(tr);
        } catch (error) {
            dropNativeObj(tr.self);
            throw error;
        }
        try {
            tr.prepare();
            tr.commit();
            dropNativeObj(tr.self);
            return r;
        } catch (error) {
            console.log(error);
            timeout = timeout - (new Date().getTime() - time);
            try {
                tr.rollback();
            } catch (error) {
                console.log("rollback fail", error)
            }
            
        }
        dropNativeObj(tr.self);
        
    }
    throw new Error("read timeout");
}

/**
 * 写数据
 * @param timeout 超时时间， 默认10毫秒， 超时将抛出异常 
*/
export const write = (mgr: Mgr, txhd: Handler, timeout = 10):any => {
    let time = new Date().getTime();
    while(timeout > 0){
        let tr = mgr.transaction(true);
        let r;
        try {
            r = txhd(tr);
        } catch (error) {
            dropNativeObj(tr.self);
            throw error;
        }
        try {
            tr.prepare();
            tr.commit();
            dropNativeObj(tr.self);
            return r;
        } catch (error) {
            timeout = timeout - (new Date().getTime() - time);
            tr.rollback();
            dropNativeObj(tr.self);
        }
    }
    throw new Error("write timeout");
}

export interface Item{
    ware: string,
    tab: string,
    key: any,
    value?: any
}

/**
 * 修改数据库的数据
 * @return Array<Item>
 */
export const modify = (tr: Tr, items: Array<Item>, lock_time: number, read_lock: boolean) => {
    let arr = [];
    for(let i = 0; i < items.length; i++){
        let it = items[i];
        let value = null;
        if(it.value !== undefined && it.value !== null){
            //if( it.tab === "config/db/User.UserId"){
                //console.log("userId1111----------------------------", it.tab, it.value);
            //}
            value = new BonBuffer().write(it.value).getBuffer();
        }
        arr.push([it.ware, it.tab, new BonBuffer().write(it.key).getBuffer(), value]);
    }
    JsDb.modify(tr, arr, lock_time, read_lock);
}

/**
 * 查询数据库的数据
 */
export const query = (tr: Tr, items: Array<Item>, lock_time: number, read_lock: boolean): Array<Item> => {
    let arr = [];
    for(let i = 0; i < items.length; i++){
        let it = items[i];
        arr.push([it.ware, it.tab, new BonBuffer().write(it.key).getBuffer()]);
    }

    return JsDb.query(tr, arr, lock_time, read_lock);
}

/**
 * 插入元信息
 */
export const alter = (tr: Tr, ware: string, tab: string, meta: TabMeta) => {
    let bb = new BonBuffer();
    if(meta){
        meta.bonEncode(bb);
    }
    JsDb.alter(tr, ware, tab, meta?bb.getBuffer():null);
}

/**
 * 插入元信息
 */
export const iterDb = (tr: Tr, ware: string, tab: string, key: any, descending:boolean, _filter:string):JsDb.DBIter => {
    if(key === null || key === undefined){
        return JsDb.iterDb(tr, ware, tab, null, descending, _filter); 
    }
    return JsDb.iterDb(tr, ware, tab, new BonBuffer().write(key).getBuffer(), descending, _filter);
}
