
import { HttpClient } from '../rust/httpc';
import { createHttpClient, HttpClientBody, HttpClientOptions, postString } from '../rust/pi_serv/js_httpc';
import {Item, DataListener} from '../../pi/db/mgr';

console.log('db_listen load!!!!!!!!!!');

export interface DbListenCfg {
    app_id: string;
    bi_ser: string;
    listen: Cfg[];
}

export interface Cfg {
    table: string,          // 表名
    getUserIdFun: Function | undefined, // 获取用户ID的方法
    values: string[],  // 获取上报值的方法
}

let dbListenCfg;

export const register_cfg = (Cfg) => {
    dbListenCfg = Cfg;
}

const dbListen: DataListener = (items: Item[]) => {
    try {
    let appID = dbListenCfg.app_id;
    let biSer = dbListenCfg.bi_ser;
    // 传输必要参数app_id、user_id
    let userDatas = new Map();
    items.forEach((item: Item) => {
        //判断是否当前数据
        let cfgs:Cfg[]  = getCfg(item.tab);
        
        cfgs.forEach((cfg) => {
            let userID = cfg.getUserIdFun?cfg.getUserIdFun(item):item.key;
            cfg.values.forEach((t) => {
                let v = item.value[t];
                let dataMap: Map<any, any> = userDatas.get(userID);
                dataMap = dataMap ? dataMap : new Map();
                dataMap.set(`${item.tab}/${t}`, v)
                userDatas.set(userID, dataMap);
            })
            
        });
    });
    if (userDatas.size === 0) {
        return
    }

    let userArr = [];
    // MAP转数组
    userDatas.forEach((v: Map<any, any>, user_id, map) => {
        let vArr = [];
        v.forEach((v2, key) => {
            vArr.push(key);
            vArr.push(v2);
        })
        userArr.push(user_id);
        userArr.push(vArr);
    })
    let arrString = `${userArr}`;
    // 发送到bi
    const client = createClient();
    addHeader(client, 'content-type', 'application/json');
    let body = {create_time: new Date().getTime(),
        app_id: appID,
        data: arrString};
    // const r = post(client, biSer, JSON.stringify(body));
    } catch (error) {
        console.log('!!!!!!!!!!!!!!!!!error:', error);
    }
    
}

const getCfg = (tab:string) : Cfg[] => {
    let listenCfg = dbListenCfg.listen;
    let rCfg = [];
    listenCfg.forEach((cfg) => {
        if(cfg.table === tab){
            rCfg.push(cfg);
        }
    });

    return rCfg;
}

/**
 * 创建客户端
 */
export const createClient = () => {
    return createHttpClient("test", HttpClientOptions.default());
};

/**
 * 添加头信息
 * 'content-type':'application/json'
 */
export const addHeader = (client: HttpClient, key: string, value: string) => {
    HttpClient.addHeaderSharedHttpc(client, key, value);
};

/**
 * jsonrpc
 * @param url -localhost + port
 * @param param -body
 */
export const post = (client: HttpClient, url: string, param: string) => {
    const body = HttpClientBody.bodyString(param);
    const result = { ok: null, err: null };
    try {
        const r = postString(client, url, body);
        if (((typeof r[1] === 'string' && (<any>r[1]).endsWith('Result is Err')) || (typeof r[1].self === 'string' && (<any>r[1].self).endsWith('Result is Err')))) {
            result.err = r[1];

            return result;
        } else {
            result.ok = r[1].text();

            return result;
        }
    } catch (e) {
        result.err = e;

        return result;
    }
};

(<any>window).dbListen = dbListen;