
import {Tr as PtTr, Mgr as PtMgr} from "../rust/pi_db/mgr"
import {DB as MemeryDb} from "../rust/pi_db/memery_db"
import {DB as FileDb} from "../rust/pi_store/lmdb_file"
import * as JsDb from "../rust/pi_serv/js_db"
import {BonBuffer} from "../../pi/util/bon"
import {TabMeta} from "../../pi/struct/sinfo"
import { dropNativeObj } from "../rust/pi_serv/js_base";
import { Handler, Mgr as PiMgr, Tr as PiTr, Item, DataListener } from "../../pi/db/mgr";
import { FileMemDB } from "../rust/pi_store/file_mem_db";

export class Mgr implements PiMgr {
    public inner: PtMgr;
    public listener: DataListener;
    constructor(inner: PtMgr, listener?: DataListener) {
        this.inner = inner;
        if (listener) {
            this.listener = listener;
        }
    }
    // 表的元信息
    public tabInfo(ware_name:string, tab_name: string): TabMeta {
        // let tabMeta = this.inner.tabInfo(ware_name, tab_name);
        // tabMeta.
        // let meta = TabMeta.bonEncode()
        
        // TODO
        return null;
    }

    // 创建事务
    public transaction(writable:boolean): Tr {
        let tr= new Tr(this.inner.transaction(writable));
        return tr;
    }

    public register<T>(ware_name: string, ware: T): boolean {
        if (ware instanceof MemeryDb) {
            return JsDb.registerMemeryDb(this.inner, ware_name, ware);
        } else if (ware instanceof FileDb) {
            return JsDb.registerFileDb(this.inner, ware_name, ware);
        } else if (ware instanceof FileMemDB) {
            return JsDb.registerFileMemDb(this.inner, ware_name, ware);
        } else {
            throw new Error("database type is not supported");
        }
    }

    /**
     * 写数据
     * @param timeout 超时时间， 默认5000毫秒， 超时将抛出异常
    */
    public write(txhd: Handler, timeout = 5000) {
        let time = new Date().getTime();
        while(timeout > 0) {
            let tr = this.transaction(true);
            let r;
            try {
                r = txhd(tr);
            } catch (error) {
                dropNativeObj(tr.inner.self);
                throw error;
            }
            try {
                tr.prepare();
                tr.commit();
                this.notify(tr.record);
                tr.record = [];
                dropNativeObj(tr.inner.self);
                return r;
            } catch (error) {
                timeout = timeout - (new Date().getTime() - time);
                tr.rollback();
                dropNativeObj(tr.inner.self);
            }
        }
        throw new Error("write timeout");
    }

    /**
     * 读数据
     * @param timeout 超时时间， 默认5000毫秒， 超时将抛出异常
    */
    public read(txhd: Handler, timeout = 5000) {
        let time = new Date().getTime();
        while(timeout > 0) {
            let tr = this.transaction(false);
            let r: any;
            try {
                r = txhd(tr);
            } catch (error) {
                dropNativeObj(tr.inner.self);
                throw error;
            }
            try {
                tr.prepare();
                tr.commit();
                dropNativeObj(tr.inner.self);
                return r;
            } catch (error) {
                console.log(error);
                timeout = timeout - (new Date().getTime() - time);
                try {
                    tr.rollback();
                } catch (error) {
                    console.log("rollback fail", error)
                }
                
            }
            dropNativeObj(tr.inner.self);
            
        }
        throw new Error("read timeout");
    }
    
    /**
	 * 读数据
	 * @param items 修改的数据
	*/
    public notify(items: Item[]) {
        if (this.listener && items.length > 0) {
            this.listener(items);
        }
    }
}

export class Tr implements PiTr {
    public inner: PtTr;
    public record: Item[];
    constructor(inner: PtTr) {
        this.inner = inner;
        this.record = [];
    }
    // 预提交
    public prepare() {
        this.inner.prepare();
    }

    // 提交
    public commit() {
        this.inner.commit();
    }

    // 回滚
    public rollback() {
        this.inner.rollback();
    }

    // 查询
    public query(items: Item[], lock_time:number,read_lock:boolean):Item[] {
        let arr = [];
        for(let i = 0; i < items.length; i++) {
            let it = items[i];
            arr.push([it.ware, it.tab, new BonBuffer().write(it.key).getBuffer()]);
        }

        return JsDb.query(this.inner, arr, lock_time, read_lock);
    }

    // 修改
    public modify(items:Item[],lock_time:number,read_lock:boolean) {
        let arr = [];
        items.forEach((item) => {
            this.record.push(item);
        });
        for(let i = 0; i < items.length; i++) {
            let it = items[i];
            let value = null;
            if(it.value !== undefined && it.value !== null) {
                // if( it.tab === "config/db/User.UserId"){
                    // console.log("userId1111----------------------------", it.tab, it.value);
                // }
                value = new BonBuffer().write(it.value).getBuffer();
            }
            arr.push([it.ware, it.tab, new BonBuffer().write(it.key).getBuffer(), value]);
        }
        JsDb.modify(this.inner, arr, lock_time, read_lock);
    }

    // 创建表
    public alter(ware_name:string,tab_name:string,meta:TabMeta) {
        let bb = new BonBuffer();
        if(meta) {
            meta.bonEncode(bb);
        }
        JsDb.alter(this.inner, ware_name, tab_name, meta?bb.getBuffer():null);
    }

    // 迭代器
    public iter<K, V>(ware: string, tab: string, key: any, descending:boolean, _filter:string): Iterable<[K, V]> {;
        if(key === null || key === undefined) {
            return {
                [Symbol.iterator]: () => new PtDbItertor1(JsDb.iterDb(this.inner, ware, tab, null, descending, _filter))
            }// iter.inner = JsDb.iterDb(tr.inner, ware, tab, null, descending, _filter); 
        } else {
            return {
                [Symbol.iterator]: () => new PtDbItertor1(JsDb.iterDb(this.inner, ware, tab, new BonBuffer().write(key).getBuffer(), descending, _filter))
            }
        }
    }

    // 迭代器
    public iter_raw<K, V>(ware: string, tab: string, key: any, descending:boolean, _filter:string): PtDbItertor<[K, V]> {;
        if(key === null || key === undefined) {
            return new PtDbItertor(JsDb.iterDb(this.inner, ware, tab, null, descending, _filter));
        } else {
            return new PtDbItertor(JsDb.iterDb(this.inner, ware, tab, new BonBuffer().write(key).getBuffer(), descending, _filter));
        }
    }
}

export class PtDbItertor1<T> implements Iterator<T> {
    public inner: JsDb.DBIter;

    constructor(inner: JsDb.DBIter) {
        this.inner = inner;
    }
    
    public next() {
        const r = this.inner.nextElem();
        if (r === null) {
            return { done: true, value: undefined };
        } else {
            return { done: false, value: r };
        }
    }
}

// 迭代器， 文件数据库不能使用foreach迭代， 因此封装了该迭代器， 直接代用next方法进行迭代
export class PtDbItertor<T> implements Iterator<T> {
    public inner: JsDb.DBIter;

    constructor(inner: JsDb.DBIter) {
        this.inner = inner;
    }
    
    public next() {
        return this.inner.nextElem();
    }
}
