/**
 * 事件服务
 */
import {getNativeObj, createNobjs, nobjsPath, addNobjListener } from "./init";
import {netConnectBind, netConnectBindTls, NetHandler, NetMgr, registerRcpListener, NetEventHandler} from "../rust/pi_serv/js_net";
import {JSGray} from "../rust/pi_serv/js_lib";
import {JSDBMonitor, registerDbJsDbMonitor} from "../rust/pi_serv/js_db";
import {NetCfg, NetCfg_Enum, RawNetCfg, TlsNetCfg} from "./server_cfg.s";
import {Entrance} from "../entrance.s";
import {NET_CONNECT_CLOSE_EVENT, NET_CONNECT_EVENT, DB_CHANGE, RPC_CONNECT_EVENT, RPC_CONNECT_CLOSE_EVENT} from "../constant";
import { createVMFactory } from "./util";
import { cfgMgr } from "../../pi/util/cfg";
import {Env} from "../../pi/lang/env";
declare var env: Env;

const Path = "pi_pt/event/event_server";

const netConnect = () => {
    let net_connect_close: string[] = []; //关闭网络连接入口
	let net_connect: string[] = []; //网络连接
	let rpc_connect_close: string[] = []; //rpc关闭网络连接入口
    let rpc_connect: string[] = []; //rpc网络连接
    let db_listen: string[] = []; //db监听
    let entranceCfg = cfgMgr.get(Entrance._$info.name);
	let netCfgs = cfgMgr.get(NetCfg._$info.name);

    if(entranceCfg){
        entranceCfg.forEach((r: Entrance, _k) => {
            if(r.note.get("event") === NET_CONNECT_EVENT){
                let file =  r.path.slice(0,r.path.lastIndexOf("."));
                net_connect.push(file + ".r.js");
            }else if(r.note.get("event") === NET_CONNECT_CLOSE_EVENT){
                let file =  r.path.slice(0,r.path.lastIndexOf("."));
                net_connect_close.push(file +  ".r.js");
            }else if(r.note.get("event") === DB_CHANGE){
                let file =  r.path.slice(0,r.path.lastIndexOf("."));
                db_listen.push(file +  ".r.js");
            }else if(r.note.get("event") === RPC_CONNECT_EVENT){
                let file =  r.path.slice(0,r.path.lastIndexOf("."));
                rpc_connect.push(file +  ".r.js");
            }else if(r.note.get("event") === RPC_CONNECT_CLOSE_EVENT){
                let file =  r.path.slice(0,r.path.lastIndexOf("."));
                rpc_connect_close.push(file +  ".r.js");
            }
        });
    }
    if (netCfgs) {
		netCfgs.forEach((net: NetCfg, _k) => {
			if (net.enum_type == NetCfg_Enum.Raw) {
				let netCfg: RawNetCfg = net.value;
				let netMgr = getNativeObj(netCfg.netMgr.name);
				if(netMgr){
					addNetHandler(netMgr, net, net_connect_close, net_connect);
				}else{
					addNobjListener(netCfg.netMgr, (netMgr: any) => {
						addNetHandler(netMgr, net, net_connect_close, net_connect);
					});
				}
			}else{
				let netCfg: TlsNetCfg = net.value as TlsNetCfg;
				let netMgr = getNativeObj(netCfg.netMgr.name);
				if(netMgr){
					addNetHandler(netMgr, net, net_connect_close, net_connect);
				}else{
					addNobjListener(netCfg.netMgr, (netMgr: any) => {
						addNetHandler(netMgr, net, net_connect_close, net_connect);
					});
				}
			}
			
		});
	}
	
	addRpcNetHandler(rpc_connect, rpc_connect_close);

	if (db_listen) {
		let dbMgr = (<any>env.dbMgr);
		//注册数据库监听器
		if(db_listen.length > 0){
			let factory = createVMFactory("__$$db_event", 128, 2, dbMgr, env.get("depend"), [Path + ".js"].concat(db_listen));
			let m = JSDBMonitor.new("_$notify", factory);
			registerDbJsDbMonitor(dbMgr.inner, m);
		}
	}
}

const addNetHandler = (netMgr: NetMgr, netCfg: NetCfg, net_connect_close: string[], net_connect: string[]) => {
    let dbMgr = (<any>env.dbMgr).inner;
    let depend = env.get("depend") as any;
    let nobjs_paths = nobjsPath(netCfg.value.nobjs);
    let factory = createVMFactory("__$$net_open_and_connect__event", 128, 2, env.dbMgr, depend, nobjs_paths.concat([Path + ".js"]).concat(net_connect));
    let factory_close =  createVMFactory("__$$net_close_event", 128, 2, env.dbMgr, depend, nobjs_paths.concat([Path + ".js"]).concat(net_connect_close));
    let gray = JSGray.new(dbMgr, factory, "net_connect$event", createNobjs(netCfg.value.nobjs));
    let gray_close = JSGray.new(dbMgr, factory_close, "net_connect_close$event", createNobjs(netCfg.value.nobjs));

    if (netCfg.enum_type == NetCfg_Enum.Raw) {
        let cfg: RawNetCfg = netCfg.value;
        netConnectBind(netMgr, cfg.addr, cfg.protocol, NetHandler.new("_$notify", gray), NetHandler.new("_$notify", gray_close));
    }else {
        let cfg: TlsNetCfg = netCfg.value as TlsNetCfg;
        netConnectBindTls(netMgr, cfg.addr, cfg.protocol, cfg.certPath, cfg.keyPath, NetHandler.new("_$notify", gray), NetHandler.new("_$notify", gray_close));
    }
}

const addRpcNetHandler = (net_connect: string[], net_connect_close: string[]) => {
	let dbMgr = (<any>env.dbMgr).inner;
	let depend = env.get("depend") as any;
	let connect_handler, close_handler;
	if (net_connect.length > 0) {
		let factory = createVMFactory("__$$rpc_net_open_and_connect__event", 128, 2, env.dbMgr, depend, net_connect);
		let gray = JSGray.new(dbMgr, factory, "rpc_net_connect$event", createNobjs([]));
		connect_handler = NetEventHandler.new("_$notify", gray);
	}
	if (net_connect_close.length > 0) {
		let factory_close =  createVMFactory("__$$rpc_net_close_event", 128, 2, env.dbMgr, depend, net_connect_close);
		let gray_close = JSGray.new(dbMgr, factory_close, "rpc_net_connect_close$event", createNobjs([]));
		close_handler = NetEventHandler.new("_$notify", gray_close);
	}
	registerRcpListener(connect_handler, close_handler);
}

netConnect();