/**
 * https服务初始化
 */
import {HttpsCfg, HttpsTlsCfg} from "./server_cfg.s";
import { cfgMgr } from "../../pi/util/cfg";
import { startHttpMount, startHttpsMount } from "../rust/https/https_impl";
import { Mount } from "../rust/https/mount";
import { StaticFile } from "../rust/https/file";
import { StaticFileBatch } from "../rust/https/files";
import { FileBatch } from "../rust/https/batch";
import { FileUpload } from "../rust/https/upload";

const initHttps = () => {
    let httpsCfgs = cfgMgr.get(HttpsCfg._$info.name);
    if(httpsCfgs){
        httpsCfgs.forEach((r: HttpsCfg,_k) => {
            let mount = Mount.new();
            let staticFile = StaticFile.newString(r.root);
			let staticFileBatch = StaticFileBatch.newString(r.root);
			let fileBatch = FileBatch.newString(r.root);
            addGenHead(staticFile,r.gen_head);
            addGenHead(staticFileBatch,r.gen_head);
            mount.mountStaticFile("/", staticFile);
			mount.mountStaticFileBatch("/files", staticFileBatch);
			mount.mountFileBatch("/batch", fileBatch);
            if(r.uploadRoot){
                mount.mountFileUpload("/upload",  FileUpload.newString(r.uploadRoot));
            }
            startHttpMount(mount, r.ip, r.port, r.keep_alive_timeout, r.handle_timeout);

        });
    }

    let httpsTlsCfgs = cfgMgr.get(HttpsTlsCfg._$info.name);
    if(httpsTlsCfgs){
        httpsTlsCfgs.forEach((r: HttpsTlsCfg,_k) => {
            let mount = Mount.new();
            let staticFile = StaticFile.newString(r.root);
			let staticFileBatch = StaticFileBatch.newString(r.root);
			let fileBatch = FileBatch.newString(r.root);
            addGenHead(staticFile,r.gen_head);
            addGenHead(staticFileBatch,r.gen_head);
            mount.mountStaticFile("/", staticFile);
			mount.mountStaticFileBatch("/files", staticFileBatch);
			mount.mountFileBatch("/batch", fileBatch);
            if(r.uploadRoot){
                mount.mountFileUpload("/upload",  FileUpload.newString(r.uploadRoot));
            }
            startHttpsMount(mount, r.ip, r.port, r.keep_alive_timeout, r.handle_timeout, r.certPath, r.keyPath);
        });
    }
    
}

const addGenHead = (f: StaticFile | StaticFileBatch, map: Map<string, string>) => {
    if(map){
        map.forEach((v, k) => {
            f.addGenRespHeader(k, v);
        });
    }
    
}

initHttps();