/**
 * mqtt服务初始化
 * 根据配置创建mqtt服务
 */

import {mqttBind, cloneServerNode, TlsNetMgr, NetMgr as RawNetMgr, mqttBindTls,globalMqttBindTcpPorts} from "../rust/pi_serv/js_net";
import {MqttCfg, NetCfg_Enum, RawNetCfg, TlsNetCfg, GlobalMqttServerCfg} from "./server_cfg.s";
import { getNativeObj, addNativeObj, addNobjListener } from "./init";
import { cfgMgr } from "../../pi/util/cfg";
import * as X from "./net.a";

const bindMqtt = () => {
    let mqttCfgs = cfgMgr.get(MqttCfg._$info.name);
    if(!mqttCfgs){
        return;
    }
    mqttCfgs.forEach((r: MqttCfg, _k) => {
		let mqttServer = getNativeObj(r.name);
        if (r.net.enum_type == NetCfg_Enum.Raw) {
            let netCfg: RawNetCfg = r.net.value;
			let netMgr = getNativeObj(netCfg.netMgr.name);
			mqttBind(mqttServer, netMgr, netCfg.addr, netCfg.protocol, r.send_buf_size, r.recv_timeout);//绑定mqtt服务
        }else {
            let netCfg: TlsNetCfg = r.net.value as TlsNetCfg;
            let netMgr = getNativeObj(netCfg.netMgr.name);
            mqttBindTls(mqttServer, netMgr, netCfg.addr, netCfg.protocol, netCfg.certPath, netCfg.keyPath, r.send_buf_size, r.recv_timeout);//绑定mqtt服务
        }
        
    });
}

const initGlobalMqtt = () => {
	let mqttCfgs = cfgMgr.get(GlobalMqttServerCfg._$info.name);
	if(!mqttCfgs){
        return;
	}
	mqttCfgs.forEach((r: GlobalMqttServerCfg, _k) => {
		globalMqttBindTcpPorts(r.ip, r.port, r.recv_buffer_size, r.send_buffer_size, r.read_buffer_capacity, r.write_buffer_capacity, r.pool_size, r.stack_size, r.timeout, r.protocol);
		return;
	});
}

bindMqtt();
initGlobalMqtt();