/**
 * rpc服务初始化
 * 根据配置，创建rpc服务，并为服务绑定handler
 */
import {RPCServer} from "../rust/rpc_tmp/server";
import { cloneServerNode, TopicHandler, cloneRpcServer} from "../rust/pi_serv/js_net";
import { ServerNode } from "../rust/mqtt_tmp/server";
import {OldRpcCfg as RpcCfg} from "./server_cfg.s";
import { getNativeObj, addNativeObj, addNobjListener, grayMgr } from "./init";
import {Entrance} from "../entrance.s";
import { cfgMgr } from "../../pi/util/cfg";
import { addOldHandler } from "./rpc";
import {Env} from "../../pi/lang/env";
declare var env: Env;

const initRpc = () => {
    let entranceCfg = cfgMgr.get(Entrance._$info.name) as Map<number, Entrance>;//rpc入口函数路径
    if(!entranceCfg){
        return;
    }
    //遍历入口配置， 将rpc的配置过滤出来
    let entrance: Entrance[] = [];
    entranceCfg.forEach((v, k) => {
        if(v.note.get("rpc") !== undefined){
            entrance.push(v);
        }
    });

    //rpc服务配置， 利用rpc服务配置创建rpc服务，并绑定handler
    let rpCfgs = cfgMgr.get(RpcCfg._$info.name) as Map<number, RpcCfg>; //rpc服务配置
    if(!rpCfgs){
        return;
    }
    let handlerMap = new Map();
    rpCfgs.forEach((rpc, _) => {
        let mqttServer = getNativeObj(rpc.mqtt.name);
        if(mqttServer){
            let rpcServer = RPCServer.new(mqttServer);
            addHandlers(rpcServer, entrance, rpc.nobjs, handlerMap);
            addNativeObj(rpc.name, rpcServer, rpc, cloneRpcServer);//注册rpc服务
        }else{
            addNobjListener(rpc.mqtt, (mqtt: ServerNode) => {
                let rpcServer = RPCServer.new(cloneServerNode(mqtt));
                addHandlers(rpcServer, entrance, rpc.nobjs, handlerMap);
                addNativeObj(rpc.name, rpcServer, rpc, cloneRpcServer);//创建rpc服务
            })
        }
    });
}

const addHandlers = (rpcServer: RPCServer, entrance: Entrance[], _nobjs: string[], map: Map<string, TopicHandler>) => {
    for(let j = 0; j < entrance.length; j++){
        addOldHandler(rpcServer, entrance[j].path, _nobjs, map, env.dbMgr, env.get("depend"), grayMgr, entrance[j].note.get("rpc"));
    }
}

initRpc();