
import {RPCServer as OldRPCServer} from "../rust/rpc_tmp/server";
import {RpcService} from "../rust/rpc/service";
import { createVMFactory, nobjsPath, createNobjs } from "./util";
import { Depend } from "../rust/pi_serv/depend";
import { registerRpcHandler, TopicHandler, arcNewTopicHandler, registerRpcTopic, RequestHandler, createRpcService } from "../rust/pi_serv/js_net";
import {JSGray, createGrayTab as createGrayTab1} from "../rust/pi_serv/js_lib";
import {Mgr} from "../../pi/db/mgr";
import { GrayTab } from "../rust/gray";
import { GrayMgr } from "../rust/pi_serv/hotfix";

const Path = "pi_pt/net/rpc_entrance";

export const addOldHandler = (rpcServer: OldRPCServer, topic: string, _nobjs: string[], map: Map<string, TopicHandler>, dbMgr: Mgr, depend: Depend, grayMgr: GrayMgr, serverName: string) => {
    let file = topic.slice(0, topic.lastIndexOf("."));
    let handler = map.get(file);
    if(!handler){
        let grayTab = grayMgr.getGrayTab(grayName(file, serverName));
        if(!grayTab){
            grayTab = createGrayTab(dbMgr, depend, file, _nobjs, grayMgr, serverName);
        }
        handler = arcNewTopicHandler(TopicHandler.new(grayTab));//创建handler， 同一js文件下的rpc函数， handler应该是同一个
        map.set(file, handler);
    }
    registerRpcHandler(rpcServer, topic, false, handler); //向rpc服务中设置handler
}

export const addHandler = (topic: string, _nobjs: string[], map: Map<string, RpcService>, dbMgr: Mgr, depend: Depend, grayMgr: GrayMgr, serverName: string) => {
	let file = topic.slice(0, topic.lastIndexOf("."));
	let service = map.get(file);
	if(!service){
		let grayTab = grayMgr.getGrayTab(grayName(file, serverName));
		if(!grayTab){
			grayTab = createGrayTab(dbMgr, depend, file, _nobjs, grayMgr, serverName);
		}
		let handler = RequestHandler.new(grayTab);
		service = createRpcService(handler);
		map.set(file, service);
	}
	registerRpcTopic(topic, service); //向rpc服务中设置topic
}

//创建灰度
export const createGrayTab = (dbMgr: Mgr, depend: Depend, file: string, _nobjs:string[], grayMgr: GrayMgr, serverName:string): GrayTab => {
    let nobjs =  createNobjs(_nobjs);
	let nobjs_paths = nobjsPath(_nobjs);
	let gray = createGrayTab1(JSGray.new((<any>dbMgr).inner, createVMFactory(file, 128, 2, dbMgr, depend, nobjs_paths.concat([Path + ".js", file + ".r.js"])), grayName(file, serverName), nobjs));
    grayMgr.addGrayTab(gray); //将灰度表添加到灰度管理器中
    return gray;
}

//更新灰度
export const updateGrayTab = (dbMgr: Mgr, depend: Depend, file: string, _nobjs:string[], grayMgr: GrayMgr, serverName: string) => {
	let nobjs_paths = nobjsPath(_nobjs);
	grayMgr.updateGray(grayName(file, serverName), (<any>dbMgr).inner, createVMFactory(serverName, 128, 2, dbMgr, depend, nobjs_paths.concat([Path + ".js", file + ".r.js"])))
}

//灰度命名
const grayName = (modName: string, serverName: string): string => {
    return modName + ".r.js$rpc$" + serverName;
}