import { Depend } from "../rust/pi_serv/depend";
import { Mgr as PtMgr} from "../rust/pi_db/mgr";
import { Mgr, Tr} from "../db/mgr";
import { GrayMgr } from "../rust/pi_serv/hotfix";
import {DEFAULT_WARE} from "../constant";
import {Entrance} from "../entrance.s";
import { cfgMgr } from "../../pi/util/cfg";
import {Env} from "../../pi/lang/env";
declare var env: UpdateEnv;

export enum Change{
    Create,
    Modify,
    Remove,
    Keep,
    NoteChange, //注解改变
}

export interface ChangeObj{
    t: Change;
    old: Entrance;
    new: Entrance;
}

export class UpdateEnv extends Env{
    oldMgr: Mgr;
    grayMgr: GrayMgr;
    diff: Map<string, Change>;
    rDiff: Map<string, Change>;

    constructor (dbMgr: PtMgr, depend: Depend, oldMgr: PtMgr, grayMgr: GrayMgr, diff?: Map<string, Change>, nobjs?: Map<string, any>){
        super();
        this.dbMgr = new Mgr(dbMgr);
        this.other = nobjs;
        this.set("depend", depend);
        this.oldMgr = new Mgr(oldMgr);
        this.grayMgr = grayMgr;
        this.diff = diff;
    }

    getOldMgr(): Mgr{
        return this.oldMgr;
    }

    getGrayMgr(): GrayMgr{
        return this.grayMgr;
    }

    getDiff(): Map<string, Change>{
        return this.diff;
    }

    getRDiff(): Map<string, Change>{
        if(this.rDiff){
            return this.rDiff;
        }
        let rdiff = new Map(); 
        this.diff.forEach((v, k) => {
            //console.log("rDiff--------------------------------", k)
            if(k.endsWith(".r.js")){
                rdiff.set(k, v);
            }
        });
        this.rDiff = rdiff;
        return rdiff;
    }
}

env = new UpdateEnv(new PtMgr((<any>self)._$new_mgr), new Depend((<any>self)._$depend), new PtMgr((<any>self)._$old_mgr), new GrayMgr((<any>self)), (<any>self)._$diff, (<any>self)._$nobjs);

// env.dbMgr = new Mgr(new PtMgr((<any>self)._$new_mgr));
// env.other = (<any>self)._$nobjs;
// env.set("depend", new Depend((<any>self)._$depend));
// env.oldMgr = new Mgr(new PtMgr((<any>self)._$old_mgr));
// env.grayMgr = new GrayMgr((<any>self));
// env.diff = (<any>self)._$diff;

delete (<any>self)._$depend;
delete (<any>self)._$new_mgr;
delete (<any>self)._$old_mgr;
delete (<any>self)._$gray_mgr;
delete (<any>self)._$diff;

export const getEnv = (): UpdateEnv => {
    return env;
}

export const readEntryCfg = (mgr: Mgr, note: string):Map<string, Entrance> => {
    let map = new Map();
    mgr.read((tr: Tr) => {
        let iter = tr.iter<string, Entrance>(DEFAULT_WARE, Entrance._$info.name, null, false, null);
        for (let n of iter) {
            let r = n[1];
            if(r && r.note.has(note)){
                map.set(r.path, r);
            }
        }
    });
    return map;
}

export const compareEntryCfg = (newMap: Map<string, Entrance>, oldMap: Map<string, Entrance>): Map<string, Map<string, ChangeObj>> => {
    let map: Map<string, Map<string, ChangeObj>> = new Map();
    oldMap.forEach((v, k) => {
        let arr = k.split(".");
        let m = map.get(arr[0]);
        if(!m){
            m = new Map();
            map.set(arr[0], m);
        }
        let n = newMap.get(k);
        if(!n){
            m.set(arr[1], {t: Change.Remove, old: v, new: null});
        }
        m.set(arr[1], {t: Change.Keep, old: v, new: n});
        oldMap.delete(k);
    });
    newMap.forEach((v, k) => {
        let arr = k.split(".");
        let m = map.get(arr[0]);
        if(!m){
            m = new Map();
            map.set(arr[0], m);
        }
        m.set(arr[1], {t: Change.Create, old: null, new: v});
    });
    return map;
}

const writeCfg = (dbMgr: Mgr) => {
    let items = [];
    cfgMgr.map.forEach((tab, k) => {
        if (k.indexOf("#") < 0) { //如果不是一个主键表， 但类型中存在主键，则不处理该数据
            let first = tab.get(0);
            if(!first){
                return; //没有数据直接返回
            }

            let primary;
            first._$getSinfo().notes && (primary = first._$getSinfo().notes.get("primary"));
            if(primary){
                return;
            }
        }
        
        tab.forEach((value, key) => {
            let ware;
            value._$getSinfo().notes && (ware = value._$getSinfo().notes.get("db") || DEFAULT_WARE);
            if(!ware){
                throw "cfg type: " + value._$getSinfo().name + " not note 'db'"; //如果配置没有db注解， 将抛出异常
            }
            items.push({ware: ware, tab: value._$getSinfo().name, key: key, value: value});
        });
    });
    if (items.length > 0) {
        dbMgr.write((tr: Tr) => {
            tr.modify(items, 1000, false);
        });
    }
}

writeCfg((<any>env.dbMgr).inner);