import {Vec} from "../rust/def/vec";
import {Mgr, Tr} from "../../pi/db/mgr";
import { tabkvNew, tabkvGetValue} from "../rust/pi_serv/js_db";
import { getDepend} from "../rust/pi_serv/js_base";
import {VMFactory} from "../rust/pi_vm/pi_vm_impl";
import {NativeObjsAuth} from "../rust/pi_vm/bonmgr";
import {BonBuffer} from "../../pi/util/bon";
import {Env} from "../../pi/lang/env";
import { Depend } from "../rust/pi_serv/depend";
import { Nobjs } from "../rust/pi_serv/js_lib";
import { cfgMgr } from "../../pi/util/cfg";
import { EntranceDependMod } from "./server_cfg.s";

declare var env: Env;

let dependModNames;
let dbListenerMod = cfgMgr.get(EntranceDependMod._$info.name) as Map<number, EntranceDependMod>;
if (dbListenerMod) {
	dbListenerMod.forEach((m, k) => {
		dependModNames = m.modNames;
		return;
	});
}

export const createVMFactory= (name: string, max_count: number, pre_count: number, mgr: Mgr, depend: Depend, file: string[]): VMFactory => {
    // todo 
    max_count = 256;
	let vmf = VMFactory.new(name, max_count, 512, 33554432, 33554432, NativeObjsAuth.withNone());
	if (dependModNames){
		for (var i = 0; i< dependModNames.length; i++) {
			file.push(dependModNames[i] + ".js");
		}
	}
    let dp = getDepend(depend, file).asSliceString();
    let map = new Map();
    let codeItems = Vec.newTabKV();

    for(let i = 0; i < dp.length; i++){
        if(map.get(dp[i])){
            continue;
        }
        map.set(dp[i], true);
        let bb = new BonBuffer();
        bb.writeUtf8(dp[i]);
        let item = tabkvNew("memory", "_$code", bb.getBuffer());
        codeItems.pushTabKV(item);
    }

    let codes = mgr.read((tr: Tr) => {
        let codeResult = (<any>tr).inner.query(codeItems, 1000, false);
        return codeResult.asSliceTabKV();
    });

    for(let i = 0; i < codes.length; i++){
        let v = tabkvGetValue(codes[i]);
        vmf = vmf.append(v);
    }
    vmf.produce(pre_count);
    return vmf;
}

export const createNobjs = (nobj_names: string[]): Nobjs => {
    let nobjs = Nobjs.new();
    if (!nobj_names) {
        return nobjs;
    }
    for (let j = 0; j < nobj_names.length; j++) {
        let obj: any = env.get(nobj_names[j]);
        nobjs.setObj(nobj_names[j], obj.self, (<any>obj).constructor._$info.name, nobj_names[j]);
    }
    return nobjs;
}

export const nobjsPath = (nobj_names: string[]): string[] => {
    let ret: string[] = [];
    for (let j = 0; j < nobj_names.length; j++) {
        let obj = env.get(nobj_names[j]);
        let r = (<any>obj).constructor._$info.name.slice(0, (<any>obj).constructor._$info.name.lastIndexOf("."));
        if(ret.indexOf(r) < 0){
            ret.push(r + ".js");
        }
    }
    return ret;
}