import {RPCClient as RpcClientNative} from "../rust/pi_serv/js_net_rpc_client";
import {Env} from "../../pi/lang/env";
import { BonBuffer, SerializeType } from '../../pi/util/bon';
import { Struct, structMgr } from '../../pi/struct/struct_mgr';
import { writeBon, read } from '../../pi/struct/util';
declare var env: Env;

export class RpcClient {
    value: RpcClientNative;

    static create = (url:string): RpcClient => { 
        let r = new RpcClient();
        r.value = RpcClientNative.create(url);
        return r;
    }

    connect = (keep_alive:number, client_id:string, timeout:number, connect_callback:(Vec) => void, close_callback: () => void) => {
        if (close_callback) {
            console.log("Closing connection callback is not supported for the time being");
        }
        this.value.connectAsync(keep_alive, client_id, timeout, null, connect_callback);
    }    
    
    request = (cmd:string, body: SerializeType, timeout:number, callback: (SerializeType) => void) => {
        let bb = new BonBuffer();
        if (body === null || body === undefined){
            bb.writeNil();
        }else if (body instanceof Struct) {
            writeBon(body, bb);
        }else{
            body.bonEncode(bb);
        }
        let c = (r: Uint8Array) => {
            if (!r) {
                callback(null);
            } else {
                let bb = new BonBuffer(r);
                callback(read(bb, structMgr));
            }
        }
        this.value.requestAsync(cmd, bb.getBuffer(), timeout, c);
    }    

    close = () => {     
        this.close();
    }
} 