/**
 * _$rpc负责rpc路由，和响应结果
 * 
 * rpc函数仅接收客户端一个参数， 并回应一个返回值， 通常情况， 一个rpc函数应该如下定义：
 * const $functionName(param: ParamName):Result => { }
 * 其中param为客户端的参数， Result为回应客户端的结果。 rpc函数也可以不接收任何参数， 如：
 * const $functionName():Result => { }
 * 此时， 客户端传来的参数将被忽略。 rpc函数也可以没有返回值, 如：
 * const $functionName() => { }, 这种情况下， 当rpc函数执行完毕后， 底层将自动回应客户端“pi/net/rpc_r.Ok”的实例。
 * 
 * 有时候， rpc函数可能需要执行一个异步的函数，才能确定其返回客户端的结果， 此时，应该如下定义rpc函数：
 * const $functionName(param: ParamName, async: any, callBack: any ) => { async.value = true }
 * 在该函数内部， 必须设置async.value为true， 以确保rpc函数结束时， 底层不会自动回应客户端， 并且， 在异步回调结束时， 应该调用callBack， 用于回应客户端结果
 * 
 * rpc函数回应客户端的类型必须是Struct， 当rpc函数内部抛出异常时， 将默认构建pi/net/rpc_r.Error的实例回应客户端
 */

declare var pi_modules;
import {Mgr as PtMgr} from "../rust/pi_db/mgr";
import {Mgr} from "../db/mgr";
import { Session as MqttSession} from "../rust/mqtt_tmp/session";
import { RpcConnect} from "../rust/rpc/connect";
import {BonBuffer} from "../../pi/util/bon";
import {Ok, Error} from "../../pi/net/rpc_r.s";
import { write, writeBon } from "../../pi/struct/util";
import { mqttRespond, rpcReply, addGlobalMqttTopic as addGlobalMqttTopic1} from "../rust/pi_serv/js_net";
import { read} from "../../pi/struct/util";
import { structMgr } from "../../pi/struct/struct_mgr";
// import {printlnU8} from "../util";
import {Session} from "./session";
import {ServerNode} from "../rust/mqtt_tmp/server";
import {setMqttTopic, unsetMqttTopic} from "../rust/pi_serv/js_net";
import {Env} from "../../pi/lang/env";
import {} from "../net/fetch"; // 定义fetch接口(https://developer.mozilla.org/zh-CN/docs/Web/API/Fetch_API)到全局
declare var env: Env;

const _$rpc_tmp = (topic: string, buffer: Uint8Array, mgr:any, session: any, nobjs: any, sokectId: number, ip: string) => {
    env = new Env();
    // console.log("_$rpc------------------------------------", topic);
    env.other = nobjs || new Map();
    // console.log(nobjs);
    let mqttSession = new MqttSession(session);
    env.dbMgr = new Mgr(new PtMgr(mgr), (<any>self).dbListen);
    env.set("mqttSession", mqttSession);
    env.set("session", new Session(sokectId));
    env.set("ip", ip);
    
    // env = new RpcEnv(new Mgr(mgr), null,  mqttSession, new Session(sokectId), nobjs);

    let index = topic.lastIndexOf(".");
    let mod = topic.slice(0, index) + ".r";
    let funName = topic.slice(index + 1, topic.length);
    let bb = new BonBuffer();
    try {
        //printlnU8(buffer);
        let param = read(new BonBuffer(buffer), structMgr);
        let async = {value: false};
        let callBack = (r: any) => {
            //rcp方法可以没有返回值， 当有返回值时， 其返回值类型必须的Struct
            try {
                if(r === undefined){
                    writeBon(new Ok(), bb);
                }else{
                    write(r, bb);
                }
                //console.log("mqttRespond------------------------------------", topic);
                mqttRespond(mqttSession, topic, bb.getBuffer());
            }catch (error) {
                console.log("rpc_return_error:" + error, "topic:", topic);
                console.log("rpc_return_error_stack:" + error.stack);
                let e = new Error();
                e.code = 0;
                e.info = error;
                writeBon(e, bb);
                mqttRespond(mqttSession, topic, bb.getBuffer());
            }
        }
        //printlnU8(buffer);
        let r = pi_modules[mod].exports[funName](param, async, callBack);
        // 如果async.value为false， 表示需要异步处理返回值
        if (async.value === false) {
            callBack(r);
        }
    } catch (error) {
        console.log("rpc_error:" + error, "topic:", topic);
        console.log("rpc_error_stack:" + error.stack);
        let e = new Error();
        e.code = 0;
        e.info = error;
        writeBon(e, bb);
        mqttRespond(mqttSession, topic, bb.getBuffer());
    }
}
(<any>self)._$rpc_tmp = _$rpc_tmp;

const _$rpc = (topic: string, buffer: Uint8Array, rid: number, mgr:any, rpcConnect: any, nobjs: any, sokectId: number, ip: string) => {
    env = new Env();
    env.other = nobjs || new Map();
    console.log(nobjs);
	let connect = new RpcConnect(rpcConnect);
    env.dbMgr = new Mgr(new PtMgr(mgr), (<any>self).dbListen);
    env.set("mqttSession", connect);
    env.set("session", new Session(sokectId));
    env.set("ip", ip);
    
    // env = new RpcEnv(new Mgr(mgr), null,  mqttSession, new Session(sokectId), nobjs);

    let index = topic.lastIndexOf(".");
    let mod = topic.slice(0, index) + ".r";
    let funName = topic.slice(index + 1, topic.length);
    let bb = new BonBuffer();
    try {
        //printlnU8(buffer);
        let param = read(new BonBuffer(buffer), structMgr);
        let async = {value: false};
        let callBack = (r: any) => {
            //rcp方法可以没有返回值， 当有返回值时， 其返回值类型必须的Struct
            try {
                if(r === undefined){
                    writeBon(new Ok(), bb);
                }else{
                    write(r, bb);
                }
				rpcReply(connect, rid, bb.getBuffer());
            }catch (error) {
                console.log("rpc_return_error:" + error, "topic:", topic);
                console.log("rpc_return_error_stack:" + error.stack);
                let e = new Error();
                e.code = 0;
                e.info = error;
                writeBon(e, bb);
                rpcReply(connect, rid, bb.getBuffer());
            }
        }
        //printlnU8(buffer);
        let r = pi_modules[mod].exports[funName](param, async, callBack);
        // 如果async.value为false， 表示需要异步处理返回值
        if (async.value === false) {
            callBack(r);
        }
    } catch (error) {
        console.log("rpc_error:" + error, "topic:", topic);
        console.log("rpc_error_stack:" + error.stack);
        let e = new Error();
        e.code = 0;
        e.info = error;
        writeBon(e, bb);
        rpcReply(connect, rid, bb.getBuffer());
    }
}
(<any>self)._$rpc = _$rpc;


(<any>ServerNode.prototype)["setTopic"] = function(topic,can_publish,can_subscribe){
    setMqttTopic(this, topic, can_publish, can_subscribe);
};

(<any>ServerNode.prototype)["unsetTopic"] = function(topic){
    unsetMqttTopic(this, topic);
};

(<any>window).addGlobalMqttTopic = addGlobalMqttTopic1;