
/**
* 基于BLS的门限签名算法
*/
import {NativeObject, call} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import * as bigInt from "../../../pi/bigint/biginteger";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {u128ToBuffer, u64ToBuffer} from "../../../pi/bigint/util";
import {Vec} from "../def/vec"
/**
* BLS算法的成员唯一id
*/
export class BlsId extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsId", 1811307610 , new Map(), []);
}
/**
* BLS算法的私钥
*/
export class BlsSecretKey extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsSecretKey", 1950901979 , new Map(), []);
}
/**
* BLS算法的公钥
*/
export class BlsPublicKey extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsPublicKey", 4133171068 , new Map(), []);
}
/**
* BLS算法的签名
*/
export class BlsSignature extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsSignature", 3585355902 , new Map(), []);
}
/**
* BLS算法的成员唯一id向量
*/
export class BlsIdVec extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsIdVec", 504915720 , new Map(), []);    
    /**
     * @param k:usize
     * @return 
     */
    static new = (k:number): BlsIdVec => {          
        let result = call(1252421489,[ k ]);     
        (<any>result) = new BlsIdVec(result);
        
        return result; 
    }
}
/**
* BLS算法私钥向量
*/
export class BlsSecKeyVec extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsSecKeyVec", 1657720652 , new Map(), []);    
    /**
     * @param k:usize
     * @return 
     */
    static new = (k:number): BlsSecKeyVec => {          
        let result = call(2592527877,[ k ]);     
        (<any>result) = new BlsSecKeyVec(result);
        
        return result; 
    }
}
/**
* BLS算法公钥向量
*/
export class BlsPubKeyVec extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsPubKeyVec", 4233471477 , new Map(), []);    
    /**
     * @param k:usize
     * @return 
     */
    static new = (k:number): BlsPubKeyVec => {          
        let result = call(3404883075,[ k ]);     
        (<any>result) = new BlsPubKeyVec(result);
        
        return result; 
    }
}
/**
* BLS算法签名向量
*/
export class BlsSigVec extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/bls.BlsSigVec", 2231627723 , new Map(), []);    
    /**
     * @param k:usize
     * @return 
     */
    static new = (k:number): BlsSigVec => {          
        let result = call(2903230657,[ k ]);     
        (<any>result) = new BlsSigVec(result);
        
        return result; 
    }
}
/**
* BLS算法初始化环境，只需要初始化一次
* @param curve BLS算法的曲线类型
* @returns 返回初始化是否成功
*/
/**
 * @param curve:Curve
 * @return bool
 */
export const blsInit = (curve:Curve): boolean => {          
    return call(2498464569,[ curve ]); 
}
/**
 * @return usize
 */
export const blsGetOpUnitSize = (): number => {     
    return call(1295262082,[  ]); 
}
/**
 * @param max_buf_size:usize
 * @return Option<String>
 */
export const blsGetCurveOrder = (max_buf_size:number): string => {          
    let result = call(2496411899,[ max_buf_size ]);     
    if(result !== undefined && result !== null){         
    }
    
    return result; 
}
/**
 * @param max_buf_size:usize
 * @return Option<String>
 */
export const blsGetFieldOrder = (max_buf_size:number): string => {          
    let result = call(755737870,[ max_buf_size ]);     
    if(result !== undefined && result !== null){         
    }
    
    return result; 
}
/**
 * @return pi_crypto::bls::BlsPublicKey
 */
export const blsGetGeneratorOfG2 = (): BlsPublicKey => {     
    let result = call(3253072797,[  ]);     
    (<any>result) = new BlsPublicKey(result);
    
    return result; 
}
/**
* 指定唯一的整数，获取BLS算法的成员唯一id
* @param x 唯一的整数
* @returns 返回BLS算法的成员唯一id
*/
/**
 * @param x:i32
 * @return pi_crypto::bls::BlsId
 */
export const blsIdSetInt = (x:number): BlsId => {          
    let result = call(4280890483,[ x ]);     
    (<any>result) = new BlsId(result);
    
    return result; 
}
/**
* 指定唯一的Dec字节串，获取BLS算法的成员唯一id
* @param buf 唯一的Dec字符串
* @returns 返回BLS算法的成员唯一id，可为空
*/
/**
 * @param buf:String
 * @return Option<pi_crypto::bls::BlsId>
 */
export const blsIdSetDecStr = (buf:string): BlsId => {          
    let result = call(2402380511,[ buf ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsId(result);
            
    }
    
    return result; 
}
/**
* 指定唯一的Hex字节串，获取BLS算法的成员唯一id
* @param buf 唯一的Hex字符串
* @returns 返回BLS算法的成员唯一id，可为空
*/
/**
 * @param buf:String
 * @return Option<pi_crypto::bls::BlsId>
 */
export const blsIdSetHexStr = (buf:string): BlsId => {          
    let result = call(2426850537,[ buf ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsId(result);
            
    }
    
    return result; 
}
/**
* 通过BLS算法的成员唯一id，获取唯一Dec字符串
* @param max_buf_size 最大的缓冲大小
* @param id BLS算法的成员唯一id
* @returns 返回唯一Dec字符串，可为空
*/
/**
 * @param max_buf_size:usize
 * @param id:&BlsId
 * @return Option<String>
 */
export const blsIdGetDecStr = (max_buf_size:number,id:BlsId): string => {               
    (<any>id) = id.self;
    
    let result = call(3075954650,[ max_buf_size,id ]);     
    if(result !== undefined && result !== null){         
    }
    
    return result; 
}
/**
* 通过BLS算法的成员唯一id，获取唯一Hex字符串
* @param max_buf_size 最大的缓冲大小
* @param id BLS算法的成员唯一id
* @returns 返回唯一Hex字符串，可为空
*/
/**
 * @param max_buf_size:usize
 * @param id:&BlsId
 * @return Option<String>
 */
export const blsIdGetHexStr = (max_buf_size:number,id:BlsId): string => {               
    (<any>id) = id.self;
    
    let result = call(3801863647,[ max_buf_size,id ]);     
    if(result !== undefined && result !== null){         
    }
    
    return result; 
}
/**
* 生成指定种子的BLS私钥
* @param buf 种子的二进制数据
* @returns 返回BLS私钥，可为空
*/
/**
 * @param buf:Vec<u8>
 * @return Option<pi_crypto::bls::BlsSecretKey>
 */
export const blsHashToSecretKey = (buf:Vec): BlsSecretKey => {          
    (<any>buf) = buf.self;
    
    let result = call(1719604587,[ buf ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSecretKey(result);
            
    }
    
    return result; 
}
/**
* 获取指定BLS私钥的公钥
* @param sec_key BLS私钥
* @returns 返回BLS公钥，可为空
*/
/**
 * @param sec_key:&BlsSecretKey
 * @return Option<pi_crypto::bls::BlsPublicKey>
 */
export const blsGetPublicKey = (sec_key:BlsSecretKey): BlsPublicKey => {          
    (<any>sec_key) = sec_key.self;
    
    let result = call(3025531400,[ sec_key ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsPublicKey(result);
            
    }
    
    return result; 
}
/**
* 获取指定BLS私钥的签名
* @param sec_key BLS私钥
* @returns 返回签名，可为空
*/
/**
 * @param sec_key:&BlsSecretKey
 * @return Option<pi_crypto::bls::BlsSignature>
 */
export const blsGetPop = (sec_key:BlsSecretKey): BlsSignature => {          
    (<any>sec_key) = sec_key.self;
    
    let result = call(3723291352,[ sec_key ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSignature(result);
            
    }
    
    return result; 
}
/**
* 验证BLS私钥的签名
* @param sig BLS私钥的签名
* @param pub_key BLS公钥
* @returns 返回验证签名是否成功
*/
/**
 * @param sig:&BlsSignature
 * @param pub_key:&BlsPublicKey
 * @return bool
 */
export const blsVerifyPop = (sig:BlsSignature,pub_key:BlsPublicKey): boolean => {          
    (<any>sig) = sig.self;
         
    (<any>pub_key) = pub_key.self;
    
    return call(1669774542,[ sig,pub_key ]); 
}
/**
* 序列化BLS算法的成员唯一id
* @param max_buf_size 最大的缓冲大小
* @param id BLS算法的成员唯一id
* @returns 返回序列化数据，可为空
*/
/**
 * @param max_buf_size:usize
 * @param id:&BlsId
 * @return Option<Vec<u8>>
 */
export const blsIdSerialize = (max_buf_size:number,id:BlsId): Vec => {               
    (<any>id) = id.self;
    
    let result = call(1235807017,[ max_buf_size,id ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new Vec(result);
            
    }
    
    return result; 
}
/**
* 序列化BLS私钥
* @param max_buf_size 最大的缓冲大小
* @param sec_key BLS私钥
* @returns 返回序列化数据，可为空
*/
/**
 * @param max_buf_size:usize
 * @param sec_key:&BlsSecretKey
 * @return Option<Vec<u8>>
 */
export const blsSecretKeySerialize = (max_buf_size:number,sec_key:BlsSecretKey): Vec => {               
    (<any>sec_key) = sec_key.self;
    
    let result = call(3671848448,[ max_buf_size,sec_key ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new Vec(result);
            
    }
    
    return result; 
}
/**
* 序列化BLS公钥
* @param max_buf_size 最大的缓冲大小
* @param pub_key BLS公钥
* @returns 返回序列化数据，可为空
*/
/**
 * @param max_buf_size:usize
 * @param pub_key:&BlsPublicKey
 * @return Option<Vec<u8>>
 */
export const blsPublicKeySerialize = (max_buf_size:number,pub_key:BlsPublicKey): Vec => {               
    (<any>pub_key) = pub_key.self;
    
    let result = call(1900424700,[ max_buf_size,pub_key ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new Vec(result);
            
    }
    
    return result; 
}
/**
* 序列化BLS签名
* @param max_buf_size 最大的缓冲大小
* @param sig BLS签名
* @returns 返回序列化数据，可为空
*/
/**
 * @param max_buf_size:usize
 * @param sig:&BlsSignature
 * @return Option<Vec<u8>>
 */
export const blsSignatureSerialize = (max_buf_size:number,sig:BlsSignature): Vec => {               
    (<any>sig) = sig.self;
    
    let result = call(2045530324,[ max_buf_size,sig ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new Vec(result);
            
    }
    
    return result; 
}
/**
* 反序列化BLS算法的成员唯一id
* @param buf 序列化数据
* @returns 返回BLS算法的成员唯一id，可为空
*/
/**
 * @param buf:Vec<u8>
 * @return Option<pi_crypto::bls::BlsId>
 */
export const blsIdDeserialize = (buf:Vec): BlsId => {          
    (<any>buf) = buf.self;
    
    let result = call(298607248,[ buf ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsId(result);
            
    }
    
    return result; 
}
/**
* 反序列化BLS私钥
* @param buf 序列化数据
* @returns 返回BLS私钥，可为空
*/
/**
 * @param buf:Vec<u8>
 * @return Option<pi_crypto::bls::BlsSecretKey>
 */
export const blsSecretKeyDeserialize = (buf:Vec): BlsSecretKey => {          
    (<any>buf) = buf.self;
    
    let result = call(2029782143,[ buf ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSecretKey(result);
            
    }
    
    return result; 
}
/**
* 反序列化BLS公钥
* @param buf 序列化数据
* @returns 返回BLS公钥，可为空
*/
/**
 * @param buf:Vec<u8>
 * @return Option<pi_crypto::bls::BlsPublicKey>
 */
export const blsPublicKeyDeserialize = (buf:Vec): BlsPublicKey => {          
    (<any>buf) = buf.self;
    
    let result = call(1922268706,[ buf ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsPublicKey(result);
            
    }
    
    return result; 
}
/**
* 反序列化BLS签名
* @param buf 序列化数据
* @returns 返回BLS签名，可为空
*/
/**
 * @param buf:Vec<u8>
 * @return Option<pi_crypto::bls::BlsSignature>
 */
export const blsSignatureDeserialize = (buf:Vec): BlsSignature => {          
    (<any>buf) = buf.self;
    
    let result = call(760927771,[ buf ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSignature(result);
            
    }
    
    return result; 
}
/**
* 检查两个指定的BLS算法的成员唯一id是否相同
* @param lhs BLS算法的成员唯一id
* @param rhs BLS算法的成员唯一id
* @returns 返回是否相同
*/
/**
 * @param lhs:&BlsId
 * @param rhs:&BlsId
 * @return bool
 */
export const blsIdIsEqual = (lhs:BlsId,rhs:BlsId): boolean => {          
    (<any>lhs) = lhs.self;
         
    (<any>rhs) = rhs.self;
    
    return call(1304117942,[ lhs,rhs ]); 
}
/**
* 检查两个指定的BLS私钥是否相同
* @param lhs BLS私钥
* @param rhs BLS私钥
* @returns 返回是否相同
*/
/**
 * @param lhs:&BlsSecretKey
 * @param rhs:&BlsSecretKey
 * @return bool
 */
export const blsSecretKeyIsEqual = (lhs:BlsSecretKey,rhs:BlsSecretKey): boolean => {          
    (<any>lhs) = lhs.self;
         
    (<any>rhs) = rhs.self;
    
    return call(1202562609,[ lhs,rhs ]); 
}
/**
* 检查两个指定的BLS公钥是否相同
* @param lhs BLS公钥
* @param rhs BLS公钥
* @returns 返回是否相同
*/
/**
 * @param lhs:&BlsPublicKey
 * @param rhs:&BlsPublicKey
 * @return bool
 */
export const blsPublicKeyIsEqual = (lhs:BlsPublicKey,rhs:BlsPublicKey): boolean => {          
    (<any>lhs) = lhs.self;
         
    (<any>rhs) = rhs.self;
    
    return call(1494397139,[ lhs,rhs ]); 
}
/**
* 检查两个指定的BLS签名是否相同
* @param lhs BLS签名
* @param rhs BLS签名
* @returns 返回是否相同
*/
/**
 * @param lhs:&BlsSignature
 * @param rhs:&BlsSignature
 * @return bool
 */
export const blsSignatureIsEqual = (lhs:BlsSignature,rhs:BlsSignature): boolean => {          
    (<any>lhs) = lhs.self;
         
    (<any>rhs) = rhs.self;
    
    return call(1251457612,[ lhs,rhs ]); 
}
/**
* 在指定的主BLS私钥上增加一个副BLS私钥
* @param sec_key 主BLS私钥
* @param rhs 副BLS私钥
*/
/**
 * @param sec_key:&BlsSecretKey
 * @param rhs:&BlsSecretKey
 */
export const blsSecretKeyAdd = (sec_key:BlsSecretKey,rhs:BlsSecretKey) => {          
    (<any>sec_key) = sec_key.self;
         
    (<any>rhs) = rhs.self;
    
    call(863200741,[ sec_key,rhs ]);
}
/**
* 在指定的主BLS公钥上增加一个副BLS公钥
* @param pub_key 主BLS公钥
* @param rhs 副BLS公钥
*/
/**
 * @param pub_key:&BlsPublicKey
 * @param rhs:&BlsPublicKey
 */
export const blsPublicKeyAdd = (pub_key:BlsPublicKey,rhs:BlsPublicKey) => {          
    (<any>pub_key) = pub_key.self;
         
    (<any>rhs) = rhs.self;
    
    call(3082139465,[ pub_key,rhs ]);
}
/**
* 在指定的主BLS主签名上增加一个副BLS签名
* @param sig 主BLS签名
* @param rhs 副BLS签名
*/
/**
 * @param sig:&BlsSignature
 * @param rhs:&BlsSignature
 */
export const blsSignatureAdd = (sig:BlsSignature,rhs:BlsSignature) => {          
    (<any>sig) = sig.self;
         
    (<any>rhs) = rhs.self;
    
    call(3576086575,[ sig,rhs ]);
}
/**
* 生成指定主私钥、共享人数、共享人的成员唯一id
* @param src_key BLS主私钥
* @param k 共享人数
* @param id BLS算法的成员唯一id
* @returns 返回共享的BLS私钥，可为空
*/
/**
 * @param src_key:&BlsSecretKey
 * @param k:usize
 * @param id:&BlsId
 * @return Option<pi_crypto::bls::BlsSecretKey>
 */
export const blsSecretKeyShare = (src_key:BlsSecretKey,k:number,id:BlsId): BlsSecretKey => {          
    (<any>src_key) = src_key.self;
              
    (<any>id) = id.self;
    
    let result = call(3750445483,[ src_key,k,id ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSecretKey(result);
            
    }
    
    return result; 
}
/**
* 生成指定主公钥、共享人数、共享人的成员唯一id
* @param src_key BLS主公钥
* @param k 共享人数
* @param id BLS算法的成员唯一id
* @returns 返回共享的BLS公钥，可为空
*/
/**
 * @param src_key:&BlsPublicKey
 * @param k:usize
 * @param id:&BlsId
 * @return Option<pi_crypto::bls::BlsPublicKey>
 */
export const blsPublicKeyShare = (src_key:BlsPublicKey,k:number,id:BlsId): BlsPublicKey => {          
    (<any>src_key) = src_key.self;
              
    (<any>id) = id.self;
    
    let result = call(3551222567,[ src_key,k,id ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsPublicKey(result);
            
    }
    
    return result; 
}
/**
* 获取成员唯一id向量中指定序号的成员唯一id
* @param vec BLS算法的成员唯一id向量
* @param index 序号
* @returns 返回BLS算法的成员唯一id，可为空
*/
/**
 * @param vec:&BlsIdVec
 * @param index:usize
 * @return Option<pi_crypto::bls::BlsId>
 */
export const blsGetIdFromVec = (vec:BlsIdVec,index:number): BlsId => {          
    (<any>vec) = vec.self;
         
    let result = call(4217857181,[ vec,index ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsId(result);
            
    }
    
    return result; 
}
/**
* 在指定的成员唯一id向量中增加指定的成员唯一id
* @param vec BLS算法的成员唯一id向量
* @param id BLS算法的成员唯一id
*/
/**
 * @param vec:&mutBlsIdVec
 * @param id:&BlsId
 */
export const blsAddIdToVec = (vec:BlsIdVec,id:BlsId) => {          
    (<any>vec) = vec.self;
         
    (<any>id) = id.self;
    
    call(3778283533,[ vec,id ]);
}
/**
* 获取私钥向量中指定序号的私钥
* @param vec BLS私钥向量
* @param index 序号
* @returns 返回BLS私钥，可为空
*/
/**
 * @param vec:&BlsSecKeyVec
 * @param index:usize
 * @return Option<pi_crypto::bls::BlsSecretKey>
 */
export const blsGetSecretKeyFromVec = (vec:BlsSecKeyVec,index:number): BlsSecretKey => {          
    (<any>vec) = vec.self;
         
    let result = call(1905417019,[ vec,index ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSecretKey(result);
            
    }
    
    return result; 
}
/**
* 在指定的私钥向量中增加指定的私钥
* @param vec BLS私钥向量
* @param sec_key BLS私钥
*/
/**
 * @param vec:&mutBlsSecKeyVec
 * @param sec_key:&BlsSecretKey
 */
export const blsAddSecretKeyToVec = (vec:BlsSecKeyVec,sec_key:BlsSecretKey) => {          
    (<any>vec) = vec.self;
         
    (<any>sec_key) = sec_key.self;
    
    call(2172313629,[ vec,sec_key ]);
}
/**
* 获取指定私钥向量的组合私钥
* @param vec BLS私钥向量
* @returns 返回组合私钥，可为空
*/
/**
 * @param vec:&BlsSecKeyVec
 * @return Option<pi_crypto::bls::BlsSecretKey>
 */
export const blsGetSecretKeyVec = (vec:BlsSecKeyVec): BlsSecretKey => {          
    (<any>vec) = vec.self;
    
    let result = call(2861556416,[ vec ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSecretKey(result);
            
    }
    
    return result; 
}
/**
* 获取公钥向量中指定序号的公钥
* @param vec BLS公钥向量
* @param index 序号
* @returns 返回BLS公钥，可为空
*/
/**
 * @param vec:&BlsPubKeyVec
 * @param index:usize
 * @return Option<pi_crypto::bls::BlsPublicKey>
 */
export const blsGetPublicKeyFromVec = (vec:BlsPubKeyVec,index:number): BlsPublicKey => {          
    (<any>vec) = vec.self;
         
    let result = call(4054179525,[ vec,index ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsPublicKey(result);
            
    }
    
    return result; 
}
/**
* 在指定的公钥向量中增加指定的公钥
* @param vec BLS公钥向量
* @param sec_key BLS公钥
*/
/**
 * @param vec:&mutBlsPubKeyVec
 * @param pub_key:&BlsPublicKey
 */
export const blsAddPublicKeyToVec = (vec:BlsPubKeyVec,pub_key:BlsPublicKey) => {          
    (<any>vec) = vec.self;
         
    (<any>pub_key) = pub_key.self;
    
    call(3718730423,[ vec,pub_key ]);
}
/**
* 获取指定公钥向量的组合公钥
* @param vec BLS公钥向量
* @returns 返回组合公钥，可为空
*/
/**
 * @param vec:&BlsPubKeyVec
 * @return Option<pi_crypto::bls::BlsPublicKey>
 */
export const blsGetPublicKeyVec = (vec:BlsPubKeyVec): BlsPublicKey => {          
    (<any>vec) = vec.self;
    
    let result = call(2864459653,[ vec ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsPublicKey(result);
            
    }
    
    return result; 
}
/**
* 获取签名向量中指定序号的签名
* @param vec BLS签名向量
* @param index 序号
* @returns 返回BLS签名，可为空
*/
/**
 * @param vec:&BlsSigVec
 * @param index:usize
 * @return Option<pi_crypto::bls::BlsSignature>
 */
export const blsGetSignatureFromVec = (vec:BlsSigVec,index:number): BlsSignature => {          
    (<any>vec) = vec.self;
         
    let result = call(376820189,[ vec,index ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSignature(result);
            
    }
    
    return result; 
}
/**
* 在指定的签名向量中增加指定的签名
* @param vec BLS签名向量
* @param sec_key BLS签名
*/
/**
 * @param vec:&mutBlsSigVec
 * @param sig:&BlsSignature
 */
export const blsAddSignatureToVec = (vec:BlsSigVec,sig:BlsSignature) => {          
    (<any>vec) = vec.self;
         
    (<any>sig) = sig.self;
    
    call(263952757,[ vec,sig ]);
}
/**
* 获取指定签名向量的组合签名
* @param vec BLS签名向量
* @returns 返回组合签名，可为空
*/
/**
 * @param vec:&BlsSigVec
 * @return Option<pi_crypto::bls::BlsSignature>
 */
export const blsGetSignatureKeyVec = (vec:BlsSigVec): BlsSignature => {          
    (<any>vec) = vec.self;
    
    let result = call(2039602097,[ vec ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSignature(result);
            
    }
    
    return result; 
}
/**
* 通过指定长度的私钥向量和成员唯一id向量，恢复主私钥
* @param sec_key_vec BLS私钥向量
* @param id_vec BLS算法的成员唯一id向量
* @param n 向量的长度
* @returns 返回主私钥，可为空
*/
/**
 * @param sec_key_vec:&BlsSecKeyVec
 * @param id_vec:&BlsIdVec
 * @param n:usize
 * @return Option<pi_crypto::bls::BlsSecretKey>
 */
export const blsSecretKeyRecover = (sec_key_vec:BlsSecKeyVec,id_vec:BlsIdVec,n:number): BlsSecretKey => {          
    (<any>sec_key_vec) = sec_key_vec.self;
         
    (<any>id_vec) = id_vec.self;
         
    let result = call(1087017908,[ sec_key_vec,id_vec,n ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSecretKey(result);
            
    }
    
    return result; 
}
/**
* 通过指定长度的公钥向量和成员唯一id向量，恢复主公钥
* @param pub_key_vec BLS公钥向量
* @param id_vec BLS算法的成员唯一id向量
* @param n 向量的长度
* @returns 返回主公钥，可为空
*/
/**
 * @param pub_key_vec:&BlsPubKeyVec
 * @param id_vec:&BlsIdVec
 * @param n:usize
 * @return Option<pi_crypto::bls::BlsPublicKey>
 */
export const blsPublicKeyRecover = (pub_key_vec:BlsPubKeyVec,id_vec:BlsIdVec,n:number): BlsPublicKey => {          
    (<any>pub_key_vec) = pub_key_vec.self;
         
    (<any>id_vec) = id_vec.self;
         
    let result = call(993477813,[ pub_key_vec,id_vec,n ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsPublicKey(result);
            
    }
    
    return result; 
}
/**
* 通过指定长度的签名向量和成员唯一id向量，恢复主签名
* @param sec_key_vec BLS签名向量
* @param id_vec BLS算法的成员唯一id向量
* @param n 向量的长度
* @returns 返回主签名，可为空
*/
/**
 * @param sig_vec:&BlsSigVec
 * @param id_vec:&BlsIdVec
 * @param n:usize
 * @return Option<pi_crypto::bls::BlsSignature>
 */
export const blsSignatureRecover = (sig_vec:BlsSigVec,id_vec:BlsIdVec,n:number): BlsSignature => {          
    (<any>sig_vec) = sig_vec.self;
         
    (<any>id_vec) = id_vec.self;
         
    let result = call(3587763353,[ sig_vec,id_vec,n ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSignature(result);
            
    }
    
    return result; 
}
/**
* BLS签名
* @param sec_key BLS私钥
* @param data 待签名的数据
* @returns 返回签名，可为空
*/
/**
 * @param sec_key:&BlsSecretKey
 * @param data:Arc<Vec<u8>>
 * @return Option<pi_crypto::bls::BlsSignature>
 */
export const blsSign = (sec_key:BlsSecretKey,data:Vec): BlsSignature => {          
    (<any>sec_key) = sec_key.self;
         
    (<any>data) = data.self;
    
    let result = call(3188209906,[ sec_key,data ]);     
    if(result !== undefined && result !== null){         
            (<any>result) = new BlsSignature(result);
            
    }
    
    return result; 
}
/**
* 验证BLS签名
* @param sig BLS签名
* @param pub_key BLS公钥
* @param data 已签名数据
* @returns 返回验证签名是否成功
*/
/**
 * @param sig:&BlsSignature
 * @param pub_key:&BlsPublicKey
 * @param data:Arc<Vec<u8>>
 * @return bool
 */
export const blsVerify = (sig:BlsSignature,pub_key:BlsPublicKey,data:Vec): boolean => {          
    (<any>sig) = sig.self;
         
    (<any>pub_key) = pub_key.self;
         
    (<any>data) = data.self;
    
    return call(2084703123,[ sig,pub_key,data ]); 
}
export enum Curve{
MclBnCurveFp254BNb=0x0,
MclBnCurveFp382_1,
MclBnCurveFp382_2,
MclBnCurveFp462,
MclBnCurveSNARK1,
MclBls12CurveFp381
}