
/**
* 基于椭圆曲线的高效数字签名算法
*/
import {NativeObject, call} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import * as bigInt from "../../../pi/bigint/biginteger";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {u128ToBuffer, u64ToBuffer} from "../../../pi/bigint/util";
import {H256} from "../hash_value"
import {H512} from "../hash_value"
/**
* 用于公钥交换
* @param public_key 待交换的公钥
* @param private_key 私钥
* @returns 返回用于交换的公钥
*/
/**
 * @param public_key:&[u8]
 * @param private_key:&[u8]
 * @return hash_value::H256
 */
export const exchange = (public_key:Uint8Array,private_key:Uint8Array): H256 => {               
    let result = call(266558349,[ public_key,private_key ]);     
    (<any>result) = new H256(result);
    
    return result; 
}
/**
* 生成密钥对
* @param seed 生成密钥的随机种子
* @returns 返回私钥和公钥
*/
/**
 * @param seed:&[u8]
 * @return (hash_value::H512,hash_value::H256)
 */
export const keypair = (seed:Uint8Array): [H512,H256] => {          
    let result = call(2282179587,[ seed ]);          
        (<any>result[0]) = new H512(result[0]);
             
        (<any>result[1]) = new H256(result[1]);
        
    return result; 
}
/**
* 签名
* @param message 待签名的数据
* @param secret_key 私钥
* @returns 返回签名
*/
/**
 * @param message:&[u8]
 * @param secret_key:&[u8]
 * @return hash_value::H512
 */
export const sign = (message:Uint8Array,secret_key:Uint8Array): H512 => {               
    let result = call(1005885597,[ message,secret_key ]);     
    (<any>result) = new H512(result);
    
    return result; 
}
/**
* 验证签名
* @param message 已签名的数据
* @param public_key 公钥
* @param signature 签名
* @returns 返回验证签名是否成功
*/
/**
 * @param message:&[u8]
 * @param public_key:&[u8]
 * @param signature:&[u8]
 * @return bool
 */
export const verify = (message:Uint8Array,public_key:Uint8Array,signature:Uint8Array): boolean => {                    
    return call(1115867356,[ message,public_key,signature ]); 
}