
/**
* 基于secp256k1的签名算法
*/
import {NativeObject, call} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import * as bigInt from "../../../pi/bigint/biginteger";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {u128ToBuffer, u64ToBuffer} from "../../../pi/bigint/util";
import {Vec} from "../def/vec"
/**
* 基于secp256k1的签名算法对象
*/
export class ECDSASecp256k1 extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/signature.ECDSASecp256k1", 2808437519 , new Map(), []);    
    /**
    
        * 构建基于secp256k1的签名算法对象
    
        * @returns 返回基于secp256k1的签名算法对象
    
        */
    /**
     * @return 
     */
    static new = (): ECDSASecp256k1 => {     
        let result = call(3054628822,[  ]);     
        (<any>result) = new ECDSASecp256k1(result);
        
        return result; 
    }    
    /**
    
        * 签名
    
        * @param msg 待签名数据，长度为32字节
    
        * @param sk 私钥，长度为32字节
    
        * @returns 返回签名
    
        */
    /**
     * @param self
     * @param msg:&[u8]
     * @param sk:&[u8]
     * @return Vec<u8>
     */
    sign = (msg:Uint8Array,sk:Uint8Array): Vec => {               
        let result = call(1378957447,[ this.self,msg,sk ]);     
        (<any>result) = new Vec(result);
        
        return result; 
    }    
    /**
    
        * 验证签名
    
        * @param msg 已签名数据，长度为32字节
    
        * @param sig 签名，长度为65~72字节
    
        * @param pk 公钥，长度为33或65字节
    
        * @returns 返回验证签名是否成功
    
        */
    /**
     * @param self
     * @param msg:&[u8]
     * @param sig:&[u8]
     * @param pk:&[u8]
     * @return bool
     */
    verify = (msg:Uint8Array,sig:Uint8Array,pk:Uint8Array): boolean => {                    
        return call(758410087,[ this.self,msg,sig,pk ]); 
    }
}
/**
* RSA签名算法对象
*/
export class Rsa extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_crypto/signature.Rsa", 1247639014 , new Map(), []);    
    /**
    
        * 从PKCS8格式的密钥数据生成RSA签名算法对象
    
        * @param input PKCS8格式的密钥数据
    
        * @returns 返回RSA签名算法对象
    
        */
    /**
     * @param input:&[u8]
     * @return pi_crypto::signature::Rsa
     */
    static fromPKCS8 = (input:Uint8Array): Rsa => {          
        let result = call(835933247,[ input ]);     
        (<any>result) = new Rsa(result);
        
        return result; 
    }    
    /**
    
        * 使用当前RSA签名算法对象和指定的RSA签名算法填充类型，对指定的数据进行签名
    
        * @param padAlg RSA签名算法填充类型
    
        * @param msg 待签名的数据
    
        * @returns 签名
    
        */
    /**
     * @param self
     * @param padAlg:PaddingAlg
     * @param msg:&[u8]
     * @return Vec<u8>
     */
    sign = (padAlg:PaddingAlg,msg:Uint8Array): Vec => {               
        let result = call(2850709748,[ this.self,padAlg,msg ]);     
        (<any>result) = new Vec(result);
        
        return result; 
    }    
    /**
    
        * 验证使用指定的RSA签名算法填充类型和指定的RSA公钥的签名
    
        * @param padAlg RSA签名算法填充类型
    
        * @param msg 已签名的数据
    
        * @param sig 签名
    
        * @param pk RSA公钥
    
        * @returns 返回验证签名是否成功
    
        */
    /**
     * @param self
     * @param padAlg:PaddingAlg
     * @param msg:&[u8]
     * @param sig:&[u8]
     * @param pk:&[u8]
     * @return bool
     */
    verify = (padAlg:PaddingAlg,msg:Uint8Array,sig:Uint8Array,pk:Uint8Array): boolean => {                         
        return call(901229592,[ this.self,padAlg,msg,sig,pk ]); 
    }
}
export enum PaddingAlg{
RSA_PKCS1_SHA256,
RSA_PKCS1_SHA384,
RSA_PKCS1_SHA512,
RSA_PSS_SHA256,
RSA_PSS_SHA384,
RSA_PSS_SHA512
}