
/**
 * 基于2pc的db管理器，每个db实现需要将自己注册到管理器上
 */
import {NativeObject, call, Error, syncCall} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {GuidGen} from "../guid"
import {TabMeta} from "./db"
import {Vec} from "../def/vec"
/**
* 表库及事务管理器
*/
export class Mgr extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_db/mgr.Mgr", 733545086 , new Map(), []);    
    /**
    
    	* 构建表库及事务管理器管理器
    
    	* @param gen 全局唯一id生成器
    
    	* @returns 返回表库及事务管理器管理器
    
    	*/
    /**
     * @param gen:GuidGen
     * @return 
     */
    static new = (gen:GuidGen): Mgr => {          
        (<any>gen) = gen.self;
        
        let result = call(4081023775,[ gen ]);     
        (<any>result) = new Mgr(result);
        
        return result; 
    }    
    /**
    
    	* 获取表的元信息
    
    	* @param ware_name 库名
    
    	* @param tab_name 表名
    
    	* @returns 返回表的元信息
    
    	*/
    /**
     * @param self
     * @param ware_name:&Atom
     * @param tab_name:&Atom
     * @return Option<Arc<pi_db::db::TabMeta>>
     */
    tabInfo = (ware_name:string,tab_name:string): TabMeta => {          
        (<any>ware_name) = ware_name;
             
        (<any>tab_name) = tab_name;
        
        let result = call(258785726,[ this.self,ware_name,tab_name ]);     
        if(result !== undefined && result !== null){         
                (<any>result) = new TabMeta(result);
                
        }
        
        return result; 
    }    
    /**
    
    	* 创建事务
    
    	* @param writable 是否为写事务
    
    	* @returns 返回事务
    
    	*/
    /**
     * @param self
     * @param writable:bool
     * @return pi_db::mgr::Tr
     */
    transaction = (writable:boolean): Tr => {          
        let result = call(951191934,[ this.self,writable ]);     
        (<any>result) = new Tr(result);
        
        return result; 
    }
}
/**
* 事务
*/
export class Tr extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_db/mgr.Tr", 761534130 , new Map(), []);    
    /**
    
    	* 预提交一个事务
    
    	* @param cb 预提交回调
    
    	* @returns 返回预提交结果
    
    	*/
    /**
     * @param self
     * @return Result<(),String>
     */
    
    prepare = ():void => {     
    
        let r = syncCall(3803008464,[ this.self ]);                          
                
        return r;
    }    
    /**
    
    	* 提交一个事务
    
    	* @param cb 提交回调
    
    	* @returns 返回提交结果
    
    	*/
    /**
     * @param self
     * @return Result<(),String>
     */
    
    commit = ():void => {     
    
        let r = syncCall(1346774966,[ this.self ]);                          
                
        return r;
    }    
    /**
    
    	* 回滚一个事务
    
    	* @param cb 回滚回调
    
    	* @returns 返回回滚结果
    
    	*/
    /**
     * @param self
     * @return Result<(),String>
     */
    
    rollback = ():void => {     
    
        let r = syncCall(977907218,[ this.self ]);                          
                
        return r;
    }    
    /**
    
    	* 查询
    
    	* @param arr 待查询的表键值条目向量
    
    	* @param lock_time 查询时的锁时长
    
    	* @param read_lock 是否读锁
    
    	* @param cb 查询回调
    
    	* @returns 查询结果
    
    	*/
    /**
     * @param self
     * @param arr:Vec<TabKV>
     * @param lock_time:Option<usize>
     * @param read_lock:bool
     * @return Result<Vec<pi_db::db::TabKV>,String>
     */
    
    query = (arr:Vec,lock_time:number,read_lock:boolean):Vec => {     
         
        (<any>arr) = arr.self;
             
        if(lock_time !== null && lock_time !== undefined){         
        }
             
        let r = syncCall(1841891766,[ this.self,arr,lock_time,read_lock ]);                          
                            (<any>r) = new Vec(r);
                            
                
        return r;
    }    
    /**
    
    	* 插入、更新或删除
    
    	* @param arr 待修改的表键值条目向量，如果值为空表示删除，如果键存在则更新，否则插入
    
    	* @param lock_time 修改时的锁时长
    
    	* @param read_lock是否读锁
    
    	* @param cb 修改回调
    
    	* @returns 修改结果
    
    	*/
    /**
     * @param self
     * @param arr:Vec<TabKV>
     * @param lock_time:Option<usize>
     * @param read_lock:bool
     * @return Result<(),String>
     */
    
    modify = (arr:Vec,lock_time:number,read_lock:boolean):void => {     
         
        (<any>arr) = arr.self;
             
        if(lock_time !== null && lock_time !== undefined){         
        }
             
        let r = syncCall(685881041,[ this.self,arr,lock_time,read_lock ]);                          
                
        return r;
    }    
    /**
    
    	* 创建、修改或删除表
    
    	* @param ware_name 库名
    
    	* @param tab_name 表名
    
    	* @param meta 表的元信息
    
    	* @param cb 更新回调
    
    	* @returns 返回更新结果
    
    	*/
    /**
     * @param self
     * @param ware_name:&Atom
     * @param tab_name:&Atom
     * @param meta:Option<Arc<TabMeta>>
     * @return Result<(),String>
     */
    
    alter = (ware_name:string,tab_name:string,meta:TabMeta):void => {     
         
        (<any>ware_name) = ware_name;
             
        (<any>tab_name) = tab_name;
             
        if(meta !== null && meta !== undefined){         
                (<any>meta) = meta.self;
                
        }
        
        let r = syncCall(3786000589,[ this.self,ware_name,tab_name,meta ]);                          
                
        return r;
    }
}