
import {NativeObject, call, syncCall, callbacks, asyncCall} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import * as bigInt from "../../../pi/bigint/biginteger";
import {u64ToBuffer, u128ToBuffer} from "../../../pi/bigint/util";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {StructInfo} from "../sinfo"
import {VMFactory} from "../pi_vm/pi_vm_impl"
import {AsyncRequestHandler} from "./js_async"
import {Vec} from "../def/vec"
import {Depend} from "./depend"
/**
* 创建一个自定义对象序列化元信息的引用计数
* @param data 元信息的二进制数据
* @returns 返回创建的结果，成功返回自定义对象序列化元信息
* @throws 失败则抛出原因描述
*/
/**
 * @param data:&[u8]
 * @return Result<Arc<sinfo::StructInfo>,bon::ReadBonErr>
 */
export const createSinfo = (data:Uint8Array): StructInfo => {          
    let result = call(1347190475,[ data ]);          
        (<any>result) = new StructInfo(result);
        
    
    return result; 
}
/**
* Copy指定的虚拟机工厂
* @param factory 待Copy的虚拟机工厂
* @returns 返回Copy的虚拟机工厂
*/
/**
 * @param factory:&VMFactory
 * @return pi_vm::pi_vm_impl::VMFactory
 */
export const cloneVmFactory = (factory:VMFactory): VMFactory => {          
    (<any>factory) = factory.self;
    
    let result = call(3993207385,[ factory ]);     
    (<any>result) = new VMFactory(result);
    
    return result; 
}
/**
* 创建一个异步请求处理器的引用计数
* @param arh 异步请求处理器
* @returns 返回异步请求处理器的引用计数
*/
/**
 * @param arh:AsyncRequestHandler
 * @return Arc<js_async::AsyncRequestHandler>
 */
export const arcNewAsyncRequestHandler = (arh:AsyncRequestHandler): AsyncRequestHandler => {          
    (<any>arh) = arh.self;
    
    let result = call(4111533257,[ arh ]);     
    (<any>result) = new AsyncRequestHandler(result);
    
    return result; 
}
/**
* 为一个指定topic的异步请求，注册处理器
* @param topic 异步请求的topic
* @param handler 异步请求处理器的引用计数
*/
/**
 * @param topic:String
 * @param handler:&Arc<AsyncRequestHandler>
 */
export const registerAsyncHandler = (topic:string,handler:AsyncRequestHandler) => {               
    (<any>handler) = handler.self;
    
    call(3272869145,[ topic,handler ]);
}
/**
* 解引用一个指定对象的引用计数
* @param v 指定对象的引用计数
* @returns 返回指定对象
*/
/**
 * @param v:&Arc<T>
 * @return &Vec<u8>
 */
export const arcDerefVec = (v:Vec): Vec => {          
    (<any>v) = v.self;
    
    let result = call(3741531906,[ v ]);     
    (<any>result) = new Vec(result);
    
    return result; 
}
/**
* 获取指定文件数组的所有依赖
* @param dp 项目依赖对象
* @param path 文件路径的数组
* @returns 所有依赖的数组
*/
/**
 * @param dp:&Depend
 * @param path:&[String]
 * @return Vec<String>
 */
export const getDepend = (dp:Depend,path:Array<string>): Vec => {          
    (<any>dp) = dp.self;
         
    let result = call(509141093,[ dp,path ]);     
    (<any>result) = new Vec(result);
    
    return result; 
}
/**
* 同步阻塞的暂停当前虚拟机的执行
* @param ms 暂停的时长，单位毫秒，等于0表示空调度当前虚拟机，大于0则表示在指定时间内暂停虚拟机运行
*/
/**
 * @param ms:u32
 */

export const sleep = (ms:number) => {     
     
    syncCall(1810043215,[ ms ]);
}
// /**
// * 同步的设置定时异步回调
// * @param ms 间隔的时长，单位毫秒
// * @param cb 异步回调
// * @returns 返回定时任务的编号
// */
// pub fn set_timeout(js: Arc<JS>, callback: u32, timeout: u32, info: Atom, args: Box<FnOnce(Arc<JS>) -> usize>) -> Option<isize> {
//     push_callback(js, callback, args, Some(timeout), info)
// }
/**
 * @param index:usize
 */
export const clearTimeout = (index:number) => {          
    call(3285798497,[ index ]);
}
/**
* 创建一个随机数生成器
* @returns 返回随机数生成器
*/
/**
 * @return js_base::Rand
 */
export const createRand = (): Rand => {     
    let result = call(59144274,[  ]);     
    (<any>result) = new Rand(result);
    
    return result; 
}
/**
* 取到一个随机32位整数
* @param or 随机数生成器
* @returns 返回随机32位整数
*/
/**
 * @param or:&mutRand
 * @return u32
 */
export const nextU32 = (or:Rand): number => {          
    (<any>or) = or.self;
    
    return call(3881780156,[ or ]); 
}
/**
* 取到一个随机64位整数
* @param or 随机数生成器
* @returns 返回随机64位整数
*/
/**
 * @param or:&mutRand
 * @return u64
 */
export const nextU64 = (or:Rand): bigInt.BigInteger => {          
    (<any>or) = or.self;
    
    let result = call(3908949488,[ or ]);     
    (<any>result) = bigInt(result);
    
    return result; 
}
/**
* 取到一个指定长度的随机buffer
* @param or 随机数生成器
* @param len 长度
* @returns 返回随机buffer
*/
/**
 * @param or:&mutRand
 * @param len:usize
 * @return Vec<u8>
 */
export const fillBytes = (or:Rand,len:number): Vec => {          
    (<any>or) = or.self;
         
    let result = call(2556550051,[ or,len ]);     
    (<any>result) = new Vec(result);
    
    return result; 
}
/**
* 取到一个指定长度的随机buffer
* @param or 随机数生成器
* @param len 长度
* @returns 返回结果，成功返回随机buffer
* @throws 失败抛出原因描述
*/
/**
 * @param or:&mutRand
 * @param len:usize
 * @return Result<Vec<u8>,String>
 */
export const tryFillBytes = (or:Rand,len:number): Vec => {          
    (<any>or) = or.self;
         
    let result = call(957759389,[ or,len ]);          
        (<any>result) = new Vec(result);
        
    
    return result; 
}
/**
 * @param t:&JSType
 * @return Result<bool,String>
 */
export const dropNativeObj = (t:any): boolean => {          
    let result = call(370495443,[ t ]);          
    
    return result; 
}
/**
 */
export const end = () => {     
    call(2041214057,[  ]);
}
/**
* 随机数生成器
*/
export class Rand extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_base.Rand", 3355421248 , new Map(), []);
}