
import {NativeObject, call, syncCall, callbacks, asyncCall} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import * as bigInt from "../../../pi/bigint/biginteger";
import {u64ToBuffer, u128ToBuffer} from "../../../pi/bigint/util";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {Vec} from "../def/vec"
import {HttpClient} from "../httpc"
import {HttpClientResponse} from "../httpc"
/**
* http客户端选项
*/
export class HttpClientOptions extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_httpc.HttpClientOptions", 1131624585 , new Map(), []);    
    /**
    
        * 创建默认http客户端选项
    
        * @returns 返回http客户端选项
    
        */
    /**
     * @return 
     */
    static default = (): HttpClientOptions => {     
        let result = call(373179692,[  ]);     
        (<any>result) = new HttpClientOptions(result);
        
        return result; 
    }    
    /**
    
        * 创建一般http客户端选项
    
        * @param https 是否使用安全的http协议
    
        * @param gzip 是否gzip压缩
    
        * @param referer 是否支持重定向
    
        * @param count 重定向最大次数
    
        * @param timeout 请求超时时长，单位毫秒
    
        * @returns 返回http客户端选项
    
        */
    /**
     * @param https:bool
     * @param gzip:bool
     * @param referer:bool
     * @param count:isize
     * @param timeout:u64
     * @return 
     */
    static normal = (https:boolean,gzip:boolean,referer:boolean,count:number,timeout:bigInt.BigInteger): HttpClientOptions => {                              
        (<any>timeout) = u64ToBuffer(timeout);
        
        let result = call(145125716,[ https,gzip,referer,count,timeout ]);     
        (<any>result) = new HttpClientOptions(result);
        
        return result; 
    }    
    /**
    
        * 创建验证主机的http客户端选项
    
        * @param cert_file 证书文件路径
    
        * @param identity_file 身份文件路径
    
        * @param pk 密钥
    
        * @param gzip 是否gzip压缩
    
        * @param referer 是否支持重定向
    
        * @param count 重定向最大次数
    
        * @param timeout 请求超时时长，单位毫秒
    
        * @returns 返回http客户端选项
    
        */
    /**
     * @param cert_file:String
     * @param identity_file:String
     * @param pk:String
     * @param gzip:bool
     * @param referer:bool
     * @param count:isize
     * @param timeout:u64
     * @return 
     */
    static vaildHost = (cert_file:string,identity_file:string,pk:string,gzip:boolean,referer:boolean,count:number,timeout:bigInt.BigInteger): HttpClientOptions => {                                        
        (<any>timeout) = u64ToBuffer(timeout);
        
        let result = call(2887071833,[ cert_file,identity_file,pk,gzip,referer,count,timeout ]);     
        (<any>result) = new HttpClientOptions(result);
        
        return result; 
    }    
    /**
    
        * 创建代理http客户端选项
    
        * @param proxy_url 代理服务器url
    
        * @param https 是否使用安全的http协议
    
        * @param gzip 是否gzip压缩
    
        * @param referer 是否支持重定向
    
        * @param count 重定向最大次数
    
        * @param timeout 请求超时时长，单位毫秒
    
        * @returns 返回http客户端选项
    
        */
    /**
     * @param proxy_url:String
     * @param https:bool
     * @param gzip:bool
     * @param referer:bool
     * @param count:isize
     * @param timeout:u64
     * @return 
     */
    static proxy = (proxy_url:string,https:boolean,gzip:boolean,referer:boolean,count:number,timeout:bigInt.BigInteger): HttpClientOptions => {                                   
        (<any>timeout) = u64ToBuffer(timeout);
        
        let result = call(2011091417,[ proxy_url,https,gzip,referer,count,timeout ]);     
        (<any>result) = new HttpClientOptions(result);
        
        return result; 
    }    
    /**
    
        * 创建验证主机的代理http客户端选项
    
        * @param cert_file 证书文件路径
    
        * @param identity_file 身份文件路径
    
        * @param pk 密钥
    
        * @param proxy_url 代理服务器url
    
        * @param gzip 是否gzip压缩
    
        * @param referer 是否支持重定向
    
        * @param count 重定向最大次数
    
        * @param timeout 请求超时时长，单位毫秒
    
        * @returns 返回http客户端选项
    
        */
    /**
     * @param cert_file:String
     * @param identity_file:String
     * @param pk:String
     * @param proxy_url:String
     * @param gzip:bool
     * @param referer:bool
     * @param count:isize
     * @param timeout:u64
     * @return 
     */
    static validHostProxy = (cert_file:string,identity_file:string,pk:string,proxy_url:string,gzip:boolean,referer:boolean,count:number,timeout:bigInt.BigInteger): HttpClientOptions => {                                             
        (<any>timeout) = u64ToBuffer(timeout);
        
        let result = call(2937777264,[ cert_file,identity_file,pk,proxy_url,gzip,referer,count,timeout ]);     
        (<any>result) = new HttpClientOptions(result);
        
        return result; 
    }
}
/**
* Http的Body
*/
export class HttpClientBody extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_httpc.HttpClientBody", 2404566785 , new Map(), []);    
    /**
    
        * 获取指定关键字的json值
    
        */
    /**
     * @param self
     * @param key:String
     * @return Option<&String>
     */
    getJsonVal = (key:string): string => {          
        let result = call(2175286088,[ this.self,key ]);     
        if(result !== undefined && result !== null){         
        }
        
        return result; 
    }    
    /**
    
        * 增加json键值对，返回键值对数量
    
        */
    /**
     * @param self
     * @param key:String
     * @param value:String
     * @return usize
     */
    addJsonKv = (key:string,value:string): number => {               
        return call(1065006446,[ this.self,key,value ]); 
    }    
    /**
    
        * 移除指定关键字的json键值对，返回被移除的值
    
        */
    /**
     * @param self
     * @param key:String
     * @return Option<String>
     */
    removeJsonKv = (key:string): string => {          
        let result = call(1500292772,[ this.self,key ]);     
        if(result !== undefined && result !== null){         
        }
        
        return result; 
    }    
    /**
    
        * 清空所有json键值对
    
        */
    /**
     * @param self
     */
    clearJsonKvs = () => {     
        call(2345066455,[ this.self ]);
    }    
    /**
    
        * 增加表单键值对
    
        */
    /**
     * @param self
     * @param key:String
     * @param value:String
     * @return 
     */
    addFormKv = (key:string,value:string): HttpClientBody => {               
        let result = call(1016322459,[ this.self,key,value ]);     
        (<any>result) = new HttpClientBody(result);
        
        return result; 
    }    
    /**
    
        * 增加表单文件
    
        */
    /**
     * @param self
     * @param key:String
     * @param file:String
     * @return Result<Self,String>
     */
    addFormFile = (key:string,file:string): HttpClientBody => {               
        let result = call(2344044784,[ this.self,key,file ]);          
            (<any>result) = new HttpClientBody(result);
            
        
        return result; 
    }    
    /**
    
        * 创建二进制的body
    
        * @param body 数据
    
        * @returns 返回Http的Body
    
        */
    /**
     * @param body:Vec<u8>
     * @return 
     */
    static bodyVec = (body:Vec): HttpClientBody => {          
        (<any>body) = body.self;
        
        let result = call(2113618061,[ body ]);     
        (<any>result) = new HttpClientBody(result);
        
        return result; 
    }    
    /**
    
        * 创建文本的body
    
        * @param body 文本
    
        * @returns 返回Http的Body
    
        */
    /**
     * @param body:String
     * @return 
     */
    static bodyString = (body:string): HttpClientBody => {          
        let result = call(794872933,[ body ]);     
        (<any>result) = new HttpClientBody(result);
        
        return result; 
    }    
    /**
    
        * 创建json的body
    
        * @param key 关键字
    
        * @param value 值
    
        * @returns 返回Http的Body
    
        */
    /**
     * @param key:Atom
     * @param value:String
     * @return 
     */
    static jsonString = (key:string,value:string): HttpClientBody => {          
        (<any>key) = key;
             
        let result = call(965054041,[ key,value ]);     
        (<any>result) = new HttpClientBody(result);
        
        return result; 
    }    
    /**
    
        * 创建表单的body
    
        * @param key 关键字
    
        * @param value 值
    
        * @returns 返回Http的Body
    
        */
    /**
     * @param key:String
     * @param value:String
     * @return 
     */
    static formString = (key:string,value:string): HttpClientBody => {               
        let result = call(2118843620,[ key,value ]);     
        (<any>result) = new HttpClientBody(result);
        
        return result; 
    }
}
/**
* 创建一个指定http客户端选项的http客户端
* @param options http客户端选项
* @returns 返回创建结果，成功返回http客户端的引用计数
* @throws 失败抛出原因描述
*/
/**
 * @param key:String
 * @param options:HttpClientOptions
 * @return Result<Arc<httpc::HttpClient>,String>
 */
export const createHttpClient = (key:string,options:HttpClientOptions): HttpClient => {               
    (<any>options) = options.self;
    
    let result = call(997239765,[ key,options ]);          
        (<any>result) = new HttpClient(result);
        
    
    return result; 
}
/**
* 使用指定http客户端向指定的url发送get请求
* @param client http客户端的引用计数
* @param url http服务器的url
* @param body Http的Body
* @param callback 发送结果的异步回调，成功返回http客户端的引用计数和http响应，失败返回原因描述
*/
/**
 * @param client:&Arc<httpc::HttpClient>
 * @param url:Atom
 * @param body:HttpClientBody<T>
 * @param (
 * @return Result<(Arc<httpc::HttpClient>,httpc::HttpClientResponse),String>
 */

export const getVecAsync = (client:HttpClient,url:string,body:HttpClientBody, callback:([HttpClient,HttpClientResponse]) => void) => {

    let callback_ = (r) => {               
            (<any>r[0]) = new HttpClient(r[0]);
                 
            (<any>r[1]) = new HttpClientResponse(r[1]);
            
    callback(r);
    };
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    asyncCall(739596726,[ client,url,body], callback_);
}

export const getVec = (client:HttpClient,url:string,body:HttpClientBody):[HttpClient,HttpClientResponse] => {     
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    let r = syncCall(2282211344,[ client,url,body ]);                                       
                                    (<any>r[0]) = new HttpClient(r[0]);
                                                 
                                    (<any>r[1]) = new HttpClientResponse(r[1]);
                                    
            
    return r;
}
/**
* 使用指定http客户端向指定的url发送get请求
* @param client http客户端的引用计数
* @param url http服务器的url
* @param body Http的Body
* @param callback 发送结果的异步回调，成功返回http客户端的引用计数和http响应，失败返回原因描述
*/
/**
 * @param client:&Arc<httpc::HttpClient>
 * @param url:Atom
 * @param body:HttpClientBody<T>
 * @param (
 * @return Result<(Arc<httpc::HttpClient>,httpc::HttpClientResponse),String>
 */

export const getStringAsync = (client:HttpClient,url:string,body:HttpClientBody, callback:([HttpClient,HttpClientResponse]) => void) => {

    let callback_ = (r) => {               
            (<any>r[0]) = new HttpClient(r[0]);
                 
            (<any>r[1]) = new HttpClientResponse(r[1]);
            
    callback(r);
    };
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    asyncCall(2173630691,[ client,url,body], callback_);
}

export const getString = (client:HttpClient,url:string,body:HttpClientBody):[HttpClient,HttpClientResponse] => {     
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    let r = syncCall(4177861558,[ client,url,body ]);                                       
                                    (<any>r[0]) = new HttpClient(r[0]);
                                                 
                                    (<any>r[1]) = new HttpClientResponse(r[1]);
                                    
            
    return r;
}
/**
* 使用指定http客户端向指定的url发送post请求
* @param client http客户端的引用计数
* @param url http服务器的url
* @param body Http的Body
* @param callback 发送结果的异步回调，成功返回http客户端的引用计数和http响应，失败返回原因描述
*/
/**
 * @param client:&Arc<httpc::HttpClient>
 * @param url:Atom
 * @param body:HttpClientBody<T>
 * @param (
 * @return Result<(Arc<httpc::HttpClient>,httpc::HttpClientResponse),String>
 */

export const postVecAsync = (client:HttpClient,url:string,body:HttpClientBody, callback:([HttpClient,HttpClientResponse]) => void) => {

    let callback_ = (r) => {               
            (<any>r[0]) = new HttpClient(r[0]);
                 
            (<any>r[1]) = new HttpClientResponse(r[1]);
            
    callback(r);
    };
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    asyncCall(1358301807,[ client,url,body], callback_);
}

export const postVec = (client:HttpClient,url:string,body:HttpClientBody):[HttpClient,HttpClientResponse] => {     
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    let r = syncCall(3729751590,[ client,url,body ]);                                       
                                    (<any>r[0]) = new HttpClient(r[0]);
                                                 
                                    (<any>r[1]) = new HttpClientResponse(r[1]);
                                    
            
    return r;
}
/**
* 使用指定http客户端向指定的url发送post请求
* @param client http客户端的引用计数
* @param url http服务器的url
* @param body Http的Body
* @param callback 发送结果的异步回调，成功返回http客户端的引用计数和http响应，失败返回原因描述
*/
/**
 * @param client:&Arc<httpc::HttpClient>
 * @param url:Atom
 * @param body:HttpClientBody<T>
 * @param (
 * @return Result<(Arc<httpc::HttpClient>,httpc::HttpClientResponse),String>
 */

export const postStringAsync = (client:HttpClient,url:string,body:HttpClientBody, callback:([HttpClient,HttpClientResponse]) => void) => {

    let callback_ = (r) => {               
            (<any>r[0]) = new HttpClient(r[0]);
                 
            (<any>r[1]) = new HttpClientResponse(r[1]);
            
    callback(r);
    };
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    asyncCall(3423707807,[ client,url,body], callback_);
}

export const postString = (client:HttpClient,url:string,body:HttpClientBody):[HttpClient,HttpClientResponse] => {     
     
    (<any>client) = client.self;
         
    (<any>url) = url;
         
    (<any>body) = body.self;
    
    let r = syncCall(2383978915,[ client,url,body ]);                                       
                                    (<any>r[0]) = new HttpClient(r[0]);
                                                 
                                    (<any>r[1]) = new HttpClientResponse(r[1]);
                                    
            
    return r;
}