
import {NativeObject, call, syncCall, callbacks, asyncCall} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import * as bigInt from "../../../pi/bigint/biginteger";
import {u64ToBuffer, u128ToBuffer} from "../../../pi/bigint/util";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {JSGray} from "./js_lib"
import {GrayTab} from "../gray"
import {ServerNode} from "../mqtt_tmp/server"
import {RPCServer} from "../rpc_tmp/server"
import {Session} from "../mqtt_tmp/session"
import {BaseService} from "../base/service"
import {RpcListener} from "../rpc/service"
import {RpcConnect} from "../rpc/connect"
import {Vec} from "../def/vec"
/**
* Tcp网络管理器
*/
export class NetMgr extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net.NetMgr", 2462173101 , new Map(), []);    
    /**
    
        * 构建Tcp网络管理器
    
        * @returns 返回Tcp网络管理器
    
        */
    /**
     * @return js_net::NetMgr
     */
    static new = (): NetMgr => {     
        let result = call(545958709,[  ]);     
        (<any>result) = new NetMgr(result);
        
        return result; 
    }
}
/**
* Tls网络管理器
*/
export class TlsNetMgr extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net.TlsNetMgr", 4120821321 , new Map(), []);    
    /**
    
        * 构建Tls网络管理器
    
        * @returns 返回Tls网络管理器
    
        */
    /**
     * @param recv_buff_size:usize
     * @return js_net::TlsNetMgr
     */
    static new = (recv_buff_size:number): TlsNetMgr => {          
        let result = call(471202658,[ recv_buff_size ]);     
        (<any>result) = new TlsNetMgr(result);
        
        return result; 
    }
}
/**
* 网络连接Handler
*/
export class NetHandler extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net.NetHandler", 1707332364 , new Map(), []);    
    /**
    
    	* 构建一个网络连接Handler
    
    	* @param handler 处理器名称
    
    	* @param gray 灰度对象
    
    	* @returns 返回网络连接Handler
    
    	*/
    /**
     * @param handler:String
     * @param gray:JSGray
     * @return js_net::NetHandler
     */
    static new = (handler:string,gray:JSGray): NetHandler => {               
        (<any>gray) = gray.self;
        
        let result = call(1849109725,[ handler,gray ]);     
        (<any>result) = new NetHandler(result);
        
        return result; 
    }
}
/**
* Topic处理器
*/
export class TopicHandler extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net.TopicHandler", 15779622 , new Map(), []);    
    /**
    
    	* 构建一个Topic处理器
    
    	* @param gray 灰度对象
    
    	* @returns 返回Topic处理器
    
    	*/
    /**
     * @param gray:&Arc<RwLock<GrayTab<JSGray>>>
     * @return 
     */
    static new = (gray:GrayTab): TopicHandler => {          
        (<any>gray) = gray.self;
        
        let result = call(2637800921,[ gray ]);     
        (<any>result) = new TopicHandler(result);
        
        return result; 
    }
}
//为pi_p2p封装一个P2PManage::new方法
// pub fn p2p_manage_new(addr: &str, arr1: Vec<String>, arr2: Vec<u32>) -> P2PManage {
//     let mut map: FnvHashMap<SocketAddr, u64> = FnvHashMap::default();
//     let mut i = 0;
//     for time in arr2 {
//         map.insert(arr1.get(i).unwrap().parse().unwrap(), time as u64);
//         i += 1;
//     }
//     P2PManage::new(addr.parse().unwrap(), map)
// }
// /**
// * 创建一个公共Socket的引用计数
// */
// pub fn creat_arc_sokect(socket: Socket ) -> Arc<Socket>{
//     Arc::new(socket)
// }
/**
* 网络事件处理器
*/
export class NetEventHandler extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net.NetEventHandler", 2899437702 , new Map(), []);    
    /**
    
        * 构建一个网络事件处理器
    
        * @param handler 处理器名称
    
        * @param gray 灰度对象
    
        * @returns 返回网络事件处理器
    
        */
    /**
     * @param handler:String
     * @param gray:JSGray
     * @return js_net::NetEventHandler
     */
    static new = (handler:string,gray:JSGray): NetEventHandler => {               
        (<any>gray) = gray.self;
        
        let result = call(3844141423,[ handler,gray ]);     
        (<any>result) = new NetEventHandler(result);
        
        return result; 
    }
}
/**
* Rpc请求处理器
*/
export class RequestHandler extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net.RequestHandler", 4208533229 , new Map(), []);    
    /**
    
        * 构建一个Rpc请求处理器
    
        * @param gray 灰度对象
    
        * @returns 返回Rpc请求处理器
    
        */
    /**
     * @param gray:&Arc<RwLock<GrayTab<JSGray>>>
     * @return 
     */
    static new = (gray:GrayTab): RequestHandler => {          
        (<any>gray) = gray.self;
        
        let result = call(3244057673,[ gray ]);     
        (<any>result) = new RequestHandler(result);
        
        return result; 
    }
}
/**
* 为mqtt绑定Tcp网络
* @param mgr Tcp网络管理器
* @param addr 绑定的地址
* @param protocol 绑定的协议名
* @param send_buf_size 发送缓冲区大小
* @param recv_timeout 接收超时时长，单位毫秒
* @returns 返回Mqtt服务器
*/
/**
 * @param server:&ServerNode
 * @param mgr:&mutNetMgr
 * @param addr:String
 * @param protocol:String
 * @param send_buf_size:usize
 * @param recv_timeout:usize
 */
export const mqttBind = (server:ServerNode,mgr:NetMgr,addr:string,protocol:string,send_buf_size:number,recv_timeout:number) => {          
    (<any>server) = server.self;
         
    (<any>mgr) = mgr.self;
                        
    call(357009886,[ server,mgr,addr,protocol,send_buf_size,recv_timeout ]);
}
/**
* 设置Tcp网络连接和关闭处理器
* @param mgr Tcp网络管理器
* @param addr 绑定的地址
* @param protocol 绑定的协议名
* @param handler 连接处理器
* @param close_handler 关闭处理器
*/
/**
 * @param mgr:&mutNetMgr
 * @param addr:String
 * @param protocol:String
 * @param handler:&NetHandler
 * @param close_handler:&NetHandler
 */
export const netConnectBind = (mgr:NetMgr,addr:string,protocol:string,handler:NetHandler,close_handler:NetHandler) => {          
    (<any>mgr) = mgr.self;
                   
    (<any>handler) = handler.self;
         
    (<any>close_handler) = close_handler.self;
    
    call(3222050891,[ mgr,addr,protocol,handler,close_handler ]);
}
/**
* 为mqtt绑定Tls网络
* @param mgr Tls网络管理器
* @param addr 绑定的地址
* @param protocol 绑定的协议名
* @param send_buf_size 发送缓冲区大小
* @param recv_timeout 接收超时时长，单位毫秒
* @returns 返回Mqtt服务器
*/
/**
 * @param server:&ServerNode
 * @param mgr:&mutTlsNetMgr
 * @param addr:String
 * @param protocol:String
 * @param cert_path:String
 * @param key_path:String
 * @param send_buf_size:usize
 * @param recv_timeout:usize
 */
export const mqttBindTls = (server:ServerNode,mgr:TlsNetMgr,addr:string,protocol:string,cert_path:string,key_path:string,send_buf_size:number,recv_timeout:number) => {          
    (<any>server) = server.self;
         
    (<any>mgr) = mgr.self;
                                  
    call(3574413612,[ server,mgr,addr,protocol,cert_path,key_path,send_buf_size,recv_timeout ]);
}
/**
* 设置Tls网络连接和关闭处理器
* @param mgr Tls网络管理器
* @param addr 绑定的地址
* @param protocol 绑定的协议名
* @param handler 连接处理器
* @param close_handler 关闭处理器
*/
/**
 * @param mgr:&mutTlsNetMgr
 * @param addr:String
 * @param protocol:String
 * @param cert_path:String
 * @param key_path:String
 * @param handler:&NetHandler
 * @param close_handler:&NetHandler
 */
export const netConnectBindTls = (mgr:TlsNetMgr,addr:string,protocol:string,cert_path:string,key_path:string,handler:NetHandler,close_handler:NetHandler) => {          
    (<any>mgr) = mgr.self;
                             
    (<any>handler) = handler.self;
         
    (<any>close_handler) = close_handler.self;
    
    call(2877879633,[ mgr,addr,protocol,cert_path,key_path,handler,close_handler ]);
}
/**
* Copy指定的Mqtt服务器
* @param node 待Copy的Mqtt服务器
* @returns 返回已Copy的Mqtt服务器
*/
/**
 * @param node:&ServerNode
 * @return mqtt_tmp::server::ServerNode
 */
export const cloneServerNode = (node:ServerNode): ServerNode => {          
    (<any>node) = node.self;
    
    let result = call(2248917003,[ node ]);     
    (<any>result) = new ServerNode(result);
    
    return result; 
}
/**
* Copy指定的RPC服务器
* @param server 待Copy的RPC服务器
* @returns 返回已Copy的RPC服务器
*/
/**
 * @param server:&RPCServer
 * @return rpc_tmp::server::RPCServer
 */
export const cloneRpcServer = (server:RPCServer): RPCServer => {          
    (<any>server) = server.self;
    
    let result = call(3695051784,[ server ]);     
    (<any>result) = new RPCServer(result);
    
    return result; 
}
/**
* 为指定的Mqtt服务器设置topic
* @param server_node Mqtt服务器
* @param topic Topic
* @param can_publish 是否可发布
* @param can_subscribe 是否可订阅
* @returns 返回是否设置成功
* @throws 失败抛出原因描述
*/
/**
 * @param server_node:&ServerNode
 * @param topic:String
 * @param can_publish:bool
 * @param can_subscribe:bool
 * @return Result<bool,String>
 */
export const setMqttTopic = (server_node:ServerNode,topic:string,can_publish:boolean,can_subscribe:boolean): boolean => {          
    (<any>server_node) = server_node.self;
                   
    let result = call(2482429183,[ server_node,topic,can_publish,can_subscribe ]);          
    
    return result; 
}
/**
* 为指定的Mqtt服务器取消topic
* @param server_node Mqtt服务器
* @param topic Topic
* @returns 返回设置结果，成功返回空
* @throws 失败抛出原因描述
*/
/**
 * @param server_node:&ServerNode
 * @param topic:String
 * @return Result<(),String>
 */
export const unsetMqttTopic = (server_node:ServerNode,topic:string): void => {          
    (<any>server_node) = server_node.self;
         
    let result = call(2867121613,[ server_node,topic ]);          
    
    return result; 
}
/**
* 发布指定topic的消息
* @param server Mqtt服务器
* @param retain 是否保留Mqtt会话
* @param qos Qos
* @param topic Topic
* @param payload 消息体
* @returns 返回发布结果，成功返回空
& @throws 失败抛出原因描述
*/
/**
 * @param server:&ServerNode
 * @param retain:bool
 * @param qos:QoS
 * @param topic:String
 * @param payload:&[u8]
 * @return Result<(),Error>
 */
export const mqttPublish = (server:ServerNode,retain:boolean,qos:QoS,topic:string,payload:Uint8Array): void => {          
    (<any>server) = server.self;
                        
    let result = call(1551231400,[ server,retain,qos,topic,payload ]);          
    
    return result; 
}
/**
* 回应指定指定topic发布的消息
* @param session Mqtt会话
* @param topic Topic
* @param data 回应的数据
*/
/**
 * @param session:&Arc<Session>
 * @param topic:String
 * @param data:&[u8]
 */
export const mqttRespond = (session:Session,topic:string,data:Uint8Array) => {          
    (<any>session) = session.self;
              
    call(2874114884,[ session,topic,data ]);
}
/**
* 为rpc注册handler
*/
/**
 * @param serv:&mutRPCServer
 * @param topic:String
 * @param sync:bool
 * @param handler:&Arc<TopicHandler>
 * @return Result<(),Error>
 */
export const registerRpcHandler = (serv:RPCServer,topic:string,sync:boolean,handler:TopicHandler): void => {          
    (<any>serv) = serv.self;
                   
    (<any>handler) = handler.self;
    
    let result = call(138660483,[ serv,topic,sync,handler ]);          
    
    return result; 
}
/**
* 创建一个Topic处理器的引用计数
*/
/**
 * @param th:TopicHandler
 * @return Arc<js_net::TopicHandler>
 */
export const arcNewTopicHandler = (th:TopicHandler): TopicHandler => {          
    (<any>th) = th.self;
    
    let result = call(527952504,[ th ]);     
    (<any>result) = new TopicHandler(result);
    
    return result; 
}
/**
* 创建Rpc服务
*/
/**
 * @param handler:&RequestHandler
 * @return Arc<base::service::BaseService>
 */
export const createRpcService = (handler:RequestHandler): BaseService => {          
    (<any>handler) = handler.self;
    
    let result = call(2333272468,[ handler ]);     
    (<any>result) = new BaseService(result);
    
    return result; 
}
/**
* 注册网络事件监听器
*/
/**
 * @param conect_handler:Option<&NetEventHandler>
 * @param close_handler:Option<&NetEventHandler>
 * @return Arc<rpc::service::RpcListener>
 */
export const registerRcpListener = (conect_handler:NetEventHandler,close_handler:NetEventHandler): RpcListener => {          
    if(conect_handler !== null && conect_handler !== undefined){
            (<any>conect_handler) = conect_handler.self;
            
    }
         
    if(close_handler !== null && close_handler !== undefined){         
            (<any>close_handler) = close_handler.self;
            
    }
	
    let result = call(466468899,[ conect_handler,close_handler ]);     
    (<any>result) = new RpcListener(result);
    
    return result; 
}
/**
* 为指定的Mqtt主题，注册指定的Rpc服务
*/
/**
 * @param topic:String
 * @param service:&Arc<BaseService>
 */
export const registerRpcTopic = (topic:string,service:BaseService) => {               
    (<any>service) = service.self;
    
    call(1703898312,[ topic,service ]);
}
/**
* rpc回应
*/
/**
 * @param connect:&Arc<RpcConnect>
 * @param rid:u32
 * @param data:&[u8]
 */
export const rpcReply = (connect:RpcConnect,rid:number,data:Uint8Array) => {          
    (<any>connect) = connect.self;
              
    call(4082873914,[ connect,rid,data ]);
}
/**
* rpc发送
*/
/**
 * @param connect:&Arc<RpcConnect>
 * @param topic:String
 * @param rid:u32
 * @param data:&[u8]
 */
export const rpcSend = (connect:RpcConnect,topic:string,rid:number,data:Uint8Array) => {          
    (<any>connect) = connect.self;
                   
    call(2617351137,[ connect,topic,rid,data ]);
}
/**
* 为指定地址的指定端口，设置指定Websocket子协议名的全局Mqtt服务器，并绑定对应的Tcp端口
*/
/**
 * @param ip:String
 * @param ports:&[u16]
 * @param recv_buffer_size:usize
 * @param send_buffer_size:usize
 * @param read_buffer_capacity:usize
 * @param write_buffer_capacity:usize
 * @param pool_size:usize
 * @param stack_size:usize
 * @param timeout:usize
 * @param protocol:String
 */
export const globalMqttBindTcpPorts = (ip:string,ports:Array<number>,recv_buffer_size:number,send_buffer_size:number,read_buffer_capacity:number,write_buffer_capacity:number,pool_size:number,stack_size:number,timeout:number,protocol:string) => {                                                       
    call(3293246594,[ ip,ports,recv_buffer_size,send_buffer_size,read_buffer_capacity,write_buffer_capacity,pool_size,stack_size,timeout,protocol ]);
}
/**
* 可以在运行时线程安全的，为全局Mqtt服务器增加指定的主题
*/
/**
 * @param is_public:bool
 * @param topic:String
 */
export const addGlobalMqttTopic = (is_public:boolean,topic:string) => {
    call(3953247239,[ is_public,topic ]);
}
/**
* 可以在运行时线程安全的，在全局Mqtt服务器上发布指定主题的消息
*/
/**
 * @param is_public:bool
 * @param topic:String
 * @param msg:Arc<Vec<u8>>
 */
export const publishGlobalMqttTopic = (is_public:boolean,topic:string,msg:Vec) => {                    
    (<any>msg) = msg.self;
    
    call(1449642520,[ is_public,topic,msg ]);
}
/*
* 可以运行时线程安全的关闭指定唯一id的Tcp连接
*/
/**
 * @param uid:usize
 * @param reason:String
 * @return bool
 */
export const closeTcpSocket = (uid:number,reason:string): boolean => {               
    return call(1428190872,[ uid,reason ]); 
}
export enum QoS{
AtMostOnce=0,
AtLeastOnce=1,
ExactlyOnce=2
}