
import {NativeObject, call, syncCall, callbacks, asyncCall} from "../../vm/vm";
import {NObject} from "../../vm/nobject";
import * as bigInt from "../../../pi/bigint/biginteger";
import {u64ToBuffer, u128ToBuffer} from "../../../pi/bigint/util";
import {StructInfo as SInfo} from "../../../pi/struct/sinfo";
import {Vec} from "../def/vec"
import {JSGray} from "./js_lib"
/**
* RPC客户端
*/
export class RPCClient extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net_rpc_client.RPCClient", 4088898725 , new Map(), []);    
    /**
    
        * 创建一个RPC客户端
    
        * @param url 需要连接的RPC服务器端url
    
        * @returns 返回创建结果，成功返回RPC客户端
    
        * @throws 失败抛出原因描述
    
        */
    /**
     * @param url:&str
     * @return Result<Self,String>
     */
    static create = (url:string): RPCClient => {          
        let result = call(1752410735,[ url ]);          
            (<any>result) = new RPCClient(result);
            
        
        return result; 
    }    
    /**
    
        * 使用RPC客户端建立一个连接
    
        * @param keep_alive 连接保持时长，单位秒
    
        * @param client_id 客户端唯一id，相同的客户端唯一id，同一时间只可创建一个连接，如已创建连接，则忽略
    
        * @param timeout 建立连接超时时长，单位秒
    
        * @param closed_handler 关闭连接的异步回调
    
        * @param connect_callback 建立连接的异步回调，成功返回空数组，失败返回原因描述
    
        */
    /**
     * @param self
     * @param keep_alive:u16
     * @param client_id:&str
     * @param timeout:u8
     * @param closed_handler:Option<CloseHandler>
     * @param (
     */
    
    connectAsync = (keep_alive:number,client_id:string,timeout:number,closed_handler:CloseHandler, connect_callback:(Vec) => void) => {
    
        let connect_callback_ = (r) => {          
            if(r !== undefined && r !== null){         
                    (<any>r) = new Vec(r);
                    
            }
            
        connect_callback(r);
        };
                        
        if(closed_handler !== null && closed_handler !== undefined){         
                (<any>closed_handler) = closed_handler.self;
                
        }
        
        asyncCall(3581032719,[ this.self,keep_alive,client_id,timeout,closed_handler], connect_callback_);
    }    
    /**
    
        * 使用RPC客户端发送请求
    
        * @param cmd Topic
    
        * @param body 数据
    
        * @param timeout 发送请求的超时时长，单位秒
    
        * @param callback 发送请求的异步回调，成功返回回应数据，失败返回原因描述
    
        */
    /**
     * @param self
     * @param cmd:String
     * @param body:&[u8]
     * @param timeout:u8
     * @param (
     */
    
    requestAsync = (cmd:string,body:Uint8Array,timeout:number, callback:(Vec) => void) => {
    
        let callback_ = (r) => {          
            if(r !== undefined && r !== null){         
                    (<any>r) = new Vec(r);
                    
            }
            
        callback(r);
        };
                   
        asyncCall(3808530099,[ this.self,cmd,body,timeout], callback_);
    }    
    /**
    
        * 关闭连接
    
        */
    /**
     * @param self
     */
    close = () => {     
        call(298029700,[ this.self ]);
    }
}
/*
* rpc 客户端连接关闭事件处理
*/
export class CloseHandler extends NObject{
    static _$info = new SInfo("pi_pt/rust/pi_serv/js_net_rpc_client.CloseHandler", 4288401962 , new Map(), []);    
    //构建一个处理器
    /**
     * @param handler:String
     * @param gray:JSGray
     * @return js_net_rpc_client::CloseHandler
     */
    static new = (handler:string,gray:JSGray): CloseHandler => {               
        (<any>gray) = gray.self;
        
        let result = call(146889029,[ handler,gray ]);     
        (<any>result) = new CloseHandler(result);
        
        return result; 
    }
}